import numpy as np

def rms(x, axis=None):
    """ Function to calculate the root mean square value of an array.
    """
    return np.sqrt(np.mean(x**2, axis=axis))   
    
def rolling_window(a, window):
    """ Efficient rolling statistics with NumPy: This is applied to Picker._statistics() to calculate statistics
        and Summary.threshold() to calcuate threshold to trigger event
        Reference from:
        http://www.rigtorp.se/2011/01/01/rolling-statistics-numpy.html
    """
    shape = a.shape[:-1] + (a.shape[-1] - window + 1, window)
    strides = a.strides + (a.strides[-1],)
    return np.lib.stride_tricks.as_strided(a, shape=shape, strides=strides) 


from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base


Base=declarative_base()

class Pick(Base):
  __tablename__="picks"
  id=Column(Integer,primary_key=True)
  sta=Column(String(5))
  chan=Column(String(3))
  net=Column(String(2))
  loc=Column(String(2))
  time=Column(DateTime)
  snr=Column(Float)
  phase=Column(String(1))
  uncert=Column(Float)
  polarity=Column(String(1))
  locate_flag=Column(Boolean)
  assoc_id=Column(Integer)
  modified_id=Column(Integer)
  t_create=Column(DateTime)
  
  def __init__(self,scnl,picktime,polarity,snr,uncert,t_create):
    self.sta=scnl.station
    self.chan=scnl.channel
    self.net=scnl.network
    self.loc=scnl.location
    self.time=picktime
    self.snr=snr
    self.uncert=uncert
    self.modified_id=None
    self.phase=None
    self.polarity=polarity
    self.locate_flag=None
    self.assoc_id=None
    self.t_create=t_create
    
  def __repr__(self):
    return "Pick <%s.%s.%s.%s %s %s %s %s>" % (self.sta,self.chan,self.net,self.loc,self.time.isoformat("T"),self.phase,self.modified_id,self.assoc_id)

class PickModified(Base):
  __tablename__="picks_modified"
  id=Column(Integer,primary_key=True)
  sta=Column(String(5))
  chan=Column(String(3))
  net=Column(String(2))
  loc=Column(String(2))
  time=Column(DateTime)
  phase=Column(String(1))
  error=Column(Float)
  locate_flag=Column(Boolean)
  assoc_id=Column(Integer)
  
  def __init__(self,sta,chan,net,loc,picktime,phase,uncert,assoc_id):
    self.sta=sta
    self.chan=chan
    self.net=net
    self.loc=loc
    self.time=picktime
    self.phase=phase
    self.uncert=uncert
    self.locate_flag=None
    self.assoc_id=assoc_id
    
  def __repr__(self):
    return "Pick <%s.%s.%s.%s %s %s %s>" % (self.sta,self.chan,self.net,self.loc,self.time.isoformat("T"),self.phase,self.assoc_id)

class Candidate(Base):
  __tablename__="candidate"
  id=Column(Integer,primary_key=True)
  ot=Column(DateTime)
  sta=Column(String(5))
  d_km=Column(Float)
  delta=Column(Float)
  weight=Column(Float)
  # P and S travel times are not completely necessary because they can be calculated but simpler to save
  tp=Column(DateTime)
  p_modified_id=Column(Integer)  # modified pick ID
  ts=Column(DateTime)
  s_modified_id=Column(Integer)  # modified pick ID
  locate_flag=Column(Boolean)
  assoc_id=Column(Integer)
  
  def __init__(self,ot,sta,d_km,delta,tp,p_modified_id,ts,s_modified_id):
    self.ot=ot
    self.sta=sta
    self.d_km=d_km
    self.delta=delta
    self.weight=None
    self.tp=tp
    self.ts=ts
    self.p_modified_id=p_modified_id
    self.s_modified_id=s_modified_id
    self.locate_flag=None
    self.assoc_id=None
  
  def __repr__(self):
    return "Candidate Event <%s %s %.2f %.2f %d %d>" % (self.ot.isoformat("T"),self.sta,self.d_km,self.delta, self.p_modified_id, self.s_modified_id)

  def __str__(self):
    return "Candidate Event <%s %s %.2f %.2f %d %d>" % (self.ot.isoformat("T"),self.sta,self.d_km,self.delta, self.p_modified_id, self.s_modified_id)

  def set_assoc_id(self,assoc_id,session,FT):
    self.assoc_id=assoc_id
    self.locate_flag=FT
    # Assign phases to modified picks
    
    # Actually only one pick_p and pick_s
    pick_p=session.query(PickModified).filter(PickModified.id==self.p_modified_id)
    for pick in pick_p:
      pick.phase='P'
      pick.assoc_id=assoc_id
      pick.locate_flag=FT
    
    pick_s=session.query(PickModified).filter(PickModified.id==self.s_modified_id)
    for pick in pick_s:
      pick.phase='S'
      pick.assoc_id=assoc_id
      pick.locate_flag=FT
    
    # Assign the phases to picks contribute to a modified picks
    picks_p=session.query(Pick).filter(Pick.modified_id==self.p_modified_id).all()
    for pick in picks_p:
      pick.phase='P'
      pick.assoc_id=assoc_id
      pick.locate_flag=FT
    
    picks_s=session.query(Pick).filter(Pick.modified_id==self.s_modified_id).all()
    for pick in picks_s:
      pick.phase='S'
      pick.assoc_id=assoc_id
      pick.locate_flag=FT  

    
class Associated(Base):
  __tablename__="associated"
  id=Column(Integer,primary_key=True)
  ot=Column(DateTime)
  ot_uncert=Column(Float)
  latitude=Column(Float)
  longitude=Column(Float)
  loc_uncert=Column(Float)
  nsta=Column(Integer)
  t_create=Column(DateTime)
  t_update=Column(DateTime)                          
  
  def __init__(self,ot,ot_uncert,latitude,longitude,loc_uncert,nsta,t_create,t_update):
    self.ot=ot
    self.ot_uncert=ot_uncert
    self.latitude=latitude
    self.longitude=longitude
    self.loc_uncert=loc_uncert
    self.nsta=nsta
    self.t_create=t_create
    self.t_update=t_update
    
  def __repr__(self):
    return "Associated Event <%s %s %.3f %.3f %d>" % (self.ot.isoformat("T"),self.ot,self.latitude,self.longitude,self.nsta)



from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base

BaseTT1D=declarative_base()

class TTtable1D(BaseTT1D):
  __tablename__='traveltimes'
  id=Column(Integer, primary_key=True)
  d_km=Column(Float)     # Distance in kilometers
  delta=Column(Float)    # Distance in degrees
  p_tt=Column(Float)     # P travel-time
  s_tt=Column(Float)     # S travel-time
  s_p=Column(Float)      # S-P time
  
  def __init__(self,d_km,delta,p_tt,s_tt,s_p):
    """ TTtable(d_km,delta,p_tt,s_tt,s_p)
    Create an entry in the travel-time table for later lookup.  The travel-time table
    is stored in a sqlalchemy data table
    d_km=Column(Float) # Distance in kilometers
    delta=Column(Float) # Distance in degrees
    p_tt=Column(Float) # P travel-time
    s_tt=Column(Float) # S travel-time
    s_p=Column(Float) # S-P time
    """
    self.d_km=d_km
    self.delta=delta
    self.p_tt=p_tt
    self.s_tt=s_tt
    self.s_p=s_p
    
  def __repr__(self):
    return "<TTtable (%.2f %.2f %.2f %.2f %.2f)>" % (self.d_km,self.delta,self.p_tt,self.s_tt,self.s_p)


class Station1D(BaseTT1D):
  __tablename__="stations"
  id=Column(Integer,primary_key=True)
  sta=Column(String(5))
  net=Column(String(2))
  loc=Column(String(2))
  latitude=Column(Float)
  longitude=Column(Float)
  elevation=Column(Float)
  starttime=Column(DateTime)
  endtime=Column(DateTime)
  
  def __init__(self,sta,net,loc,latitude,longitude,elevation):
    self.sta=sta
    self.net=net
    self.loc=loc
    self.latitude=latitude
    self.longitude=longitude
    self.elevation=elevation
    self.starttime=None
    self.endtime=None
  
  def __repr__(self):
    return "Station <%s.%s.%s.%s %s %s %s>" % (self.sta,self.net,self.loc,self.latitude,self.longitude,self.starttime,self.endtime)    
  


class SCNL():
  """ This class helps us work with SCNL definitions and compare and go between different
  representations"""
  def __init__(self,input=None):
#    print input,type(input)
    if not isinstance(input, SCNL):
      self.station=None
      self.channel=None
      self.network=None
      self.location=None
    if type(input) is str:
      self.parse_scnlstr(input)
    if type(input) is list:
      if len(input)==4:
        self.station,self.channel,self.network,self.location=input
      if len(input)==3:
        self.station,self.channel,self.network=input
      if len(input)<3:
        raise SCNL_InputError("List input has %d fields minimum of 3 required" % (len(input)))
    
  def __repr__(self):
    return self.__str__()
    
  def __str__(self):
    return "%s.%s.%s.%s" % (self.station,self.channel,self.network,self.location)
  
  def to_winston(self):
    if self.location==None or self.location=='--':
      return "%s$%s$%s" % (self.station,self.channel,self.network)
    else:
      return "%s$%s$%s$%s" % (self.station,self.channel,self.network,self.location)
      
  def to_ewscnl(self):
    if self.location==None or self.location=='--':
      return "%s.%s.%s.--" % (self.station,self.channel,self.network)
    else:
      return "%s.%s.%s.%s" % (self.station,self.channel,self.network,self.location)
      
  def to_seed(self):
    if self.location==None or self.location=='--':
      return "%s.%s.%s.." % (self.station,self.channel,self.network)
    else:
      return "%s.%s.%s.%s." % (self.station,self.channel,self.network,self.location) 
           
  def parse_scnlstr(self,scnl_str):
    if re.search('\.',scnl_str):
      # Looks like an earthworm delimited scnl
      self.from_ew(scnl_str)
    if re.search('\$',scnl_str):
      # Looks like a winston scnl
      self.from_winston(scnl_str)

  def from_ew(self,scnl_str):
    scnl=scnl_str.split('.')
    self.station=scnl[0]
    self.channel=scnl[1]
    self.network=scnl[2]
    self.location=scnl[3]
  
  def from_winston(self,scnl_str):
    scnl=scnl_str.split('$')
    self.station=scnl[0]
    self.channel=scnl[1]
    self.network=scnl[2]
    if len(scnl)==4:
      self.location=scnl[3]
    else:
      self.location=None
      
    def __eq__(self,y):
      if type(y) is str:
        tmp=y
        y=SCNL()
        y.parse_scnlstr(tmp)
      if isinstance(y,SCNL):     
        if self.to_ewscnl==y.to_ewscnl:
          return True
        else:
          return False
      else: # We were given something we can't compare
        return False

    def __neq__(self,y):
      if type(y) is str:
        tmp=y
        y=SCNL()
        y.parse_scnlstr(tmp)
      if isinstance(y,SCNL):     
        if self.to_ewscnl!=y.to_ewscnl:
          return True
        else:
          return False
      else: # We were given something we can't compare
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
set of functions to drive EasyQuake
"""
#import sys
#sys.path.append("/home/jwalter/syncpython")
from .phasepapy import fbpicker
pathgpd = '/'.join(str(fbpicker.__file__).split("/")[:-2])+'/gpd_predict'

from .phasepapy import tables1D, assoc1D
from .phasepapy import tt_stations_1D
from obspy import UTCDateTime
from obspy import Inventory, read_inventory
from obspy.clients.fdsn import Client
from obspy import read
import datetime
import numpy as np
import glob
import sys
import os
import stat
st = os.stat(pathgpd+'/gpd_predict.py')
os.chmod(pathgpd+'/gpd_predict.py', st.st_mode | stat.S_IEXEC)
import obspy.taup as taup
from obspy import geodetics
from obspy.clients.fdsn.mass_downloader import CircularDomain, RectangularDomain, Restrictions, MassDownloader
from obspy.core.event.base import WaveformStreamID
from obspy.core.event import ResourceIdentifier
from sqlalchemy.orm import *
from sqlalchemy import create_engine
import numpy as np
import obspy.core as oc
import pandas as pd
import sqlite3
from sqlite3 import Error
from obspy.geodetics import gps2dist_azimuth

import pylab as plt
import re
from datetime import datetime
from mpl_toolkits.basemap import Basemap


from sqlalchemy import create_engine
from obspy import Stream
from obspy.core.event import Catalog, Event, Magnitude, Origin, Pick, StationMagnitude, Amplitude, Arrival
from obspy.signal.invsim import simulate_seismometer as seis_sim
fmtP = "%4s%1sP%1s%1i %15s"
fmtS = "%12s%1sS%1s%1i\n"



fmt = "%6s%02i%05.2f%1s%03i%05.2f%1s%4i\n"


#min_proba = 0.993 # Minimum softmax probability for phase detection
## try 0.992 if you have the computing power
#freq_min = 3.0
#freq_max = 20.0
#filter_data = True
#decimate_data = True # If false, assumes data is already 100 Hz samprate
#n_shift = 10 # Number of samples to shift the sliding window at a time
#n_gpu = 1 # Number of GPUs to use (if any)
######################
#batch_size = 1000*3
#
#half_dur = 2.00
#only_dt = 0.01
#n_win = int(half_dur/only_dt)
#n_feat = 2*n_win


from datetime import timedelta, date

def daterange(start_date, end_date):
    for n in range(int ((end_date - start_date).days)):
        yield start_date + timedelta(n)

class SCNL():
    """ This class is copied from PhasePaPy"""
    def __init__(self,input=None):
        if not isinstance(input, SCNL):
            self.station=None
            self.channel=None
            self.network=None
            self.location=None
        if type(input) is str:
            self.parse_scnlstr(input)
        if type(input) is list:
            if len(input)==4:
                self.station,self.channel,self.network,self.location=input
            if len(input)==3:
                self.station,self.channel,self.network=input
                

def download_mseed(dirname=None, project_folder=None, single_date=None, minlat=None, maxlat=None, minlon=None, maxlon=None):
    starting = UTCDateTime(single_date.strftime("%Y")+'-'+single_date.strftime("%m")+'-'+single_date.strftime("%d")+'T00:00:00.0')
    stopping = starting + 86430
    starttime = starting
    endtime = stopping
    #domain = CircularDomain(-90,0,minradius=0.0, maxradius=30.0)
    domain = RectangularDomain(minlatitude=minlat, maxlatitude=maxlat,minlongitude=minlon, maxlongitude=maxlon)
    #domain = RectangularDomain(minlatitude=-90, maxlatitude=-60,minlongitude=-180, maxlongitude=180)
    restrictions = Restrictions(starttime=starttime, endtime=endtime,reject_channels_with_gaps=False,minimum_length=0,minimum_interstation_distance_in_m=1000, channel_priorities=["[BHSE]H[NE12Z]"])
    mseed1 = project_folder+'/'+dirname
    if not os.path.exists(mseed1):
        os.makedirs(mseed1) #domain = CircularDomain(-90,0,minradius=0.0, maxradius=30.0)
    #original1 = project_folder+'/*.[BH]??__'+dirname+'*'
    #os.system("mv %s %s" % (original1,mseed1))
    mdl = MassDownloader()
    mdl.download(domain, restrictions, threads_per_client=4, mseed_storage=mseed1,stationxml_storage=mseed1)

def download_mseed_event(dirname=None, project_folder=None, starting=None, stopping = None, minlat=None, maxlat=None, minlon=None, maxlon=None, maxrad=None):
    starttime = starting
    endtime = stopping
    #domain = CircularDomain(lat1,lon1,minradius=0.0, maxradius=maxrad)
    domain = RectangularDomain(minlatitude=minlat, maxlatitude=maxlat,minlongitude=minlon, maxlongitude=maxlon)
    #domain = RectangularDomain(minlatitude=-90, maxlatitude=-60,minlongitude=-180, maxlongitude=180)
    restrictions = Restrictions(starttime=starttime, endtime=endtime,chunklength_in_sec=86400,reject_channels_with_gaps=False,minimum_length=0,minimum_interstation_distance_in_m=1000, channel_priorities=["[BHSE]H[NE12Z]"])
    mseed1 = project_folder+'/'+dirname
    if not os.path.exists(mseed1):
        os.makedirs(mseed1) #domain = CircularDomain(-90,0,minradius=0.0, maxradius=30.0)
    #original1 = project_folder+'/*.[BH]??__'+dirname+'*'
    #os.system("mv %s %s" % (original1,mseed1))
    mdl = MassDownloader()
    mdl.download(domain, restrictions, threads_per_client=4, mseed_storage=mseed1,stationxml_storage=mseed1)
    
def download_mseed_event_radial(dirname=None, project_folder=None, starting=None, stopping = None, lat1=None, lon1=None, maxrad=None):
    starttime = starting
    endtime = stopping
    domain = CircularDomain(lat1,lon1,minradius=0.0, maxradius=maxrad)
    #domain = RectangularDomain(minlatitude=minlat, maxlatitude=maxlat,minlongitude=minlon, maxlongitude=maxlon)
    #domain = RectangularDomain(minlatitude=-90, maxlatitude=-60,minlongitude=-180, maxlongitude=180)
    restrictions = Restrictions(starttime=starttime, endtime=endtime,chunklength_in_sec=86400,reject_channels_with_gaps=False,minimum_length=0,minimum_interstation_distance_in_m=1000, channel_priorities=["[BHSE]H[NE12Z]"])
    mseed1 = project_folder+'/'+dirname
    if not os.path.exists(mseed1):
        os.makedirs(mseed1) #domain = CircularDomain(-90,0,minradius=0.0, maxradius=30.0)
    #original1 = project_folder+'/*.[BH]??__'+dirname+'*'
    #os.system("mv %s %s" % (original1,mseed1))
    mdl = MassDownloader()
    mdl.download(domain, restrictions, threads_per_client=4, mseed_storage=mseed1,stationxml_storage=mseed1)

def process_local_sac():
    print('Local sac files')


                
def build_tt_tables(lat1=None,long1=None,maxrad=None,starting=None, stopping=None, channel_codes=['EH','BH','HH'],db=None,maxdist=500.,source_depth=5.):
    """ 
    """
    # Create a connection to an sqlalchemy database
    tt_engine=create_engine(db,echo=False)
    tt_stations_1D.BaseTT1D.metadata.create_all(tt_engine)
    TTSession=sessionmaker(bind=tt_engine)
    tt_session=TTSession()
    fdsnclient=Client()
    inv=fdsnclient.get_stations(starttime=starting,endtime=stopping,latitude=lat1,longitude=long1,maxradius=maxrad,channel='*HZ',level='channel')
    # Get inventory
    for net in inv:
        network=net.code
        for sta in net:
            loccodes=[]
            for ch in sta:
                for cc in channel_codes:
                  if re.match(cc,ch.code):
                    if not ch.location_code in loccodes:
                      loccodes.append(ch.location_code)
            for loc in loccodes:
                print(sta.code,network,loc,sta.latitude,sta.longitude,sta.elevation)
                station=tt_stations_1D.Station1D(sta.code,network,loc,sta.latitude,sta.longitude,sta.elevation)
                tt_session.add(station)
            tt_session.commit()

    # Now we have to build our traveltime lookup tables
    # We will use IASP91 here but obspy.taup does let you build your own model
    velmod=taup.TauPyModel(model='iasp91')
    delta_distance=1. # km for spacing tt calculations  
    distance_km=np.arange(0,maxdist+delta_distance,delta_distance)
    for d_km in distance_km:
        d_deg=geodetics.kilometer2degrees(d_km)
        ptimes=[]
        stimes=[]
        p_arrivals=velmod.get_travel_times(source_depth_in_km=source_depth,
          distance_in_degree=d_deg,phase_list=['P','p'])
        for p in p_arrivals:
            ptimes.append(p.time)
        s_arrivals=velmod.get_travel_times(source_depth_in_km=source_depth,
            distance_in_degree=d_deg,phase_list=['S','s'])
        for s in s_arrivals:
            stimes.append(s.time)
        tt_entry=tt_stations_1D.TTtable1D(d_km,d_deg,np.min(ptimes),np.min(stimes),np.min(stimes)-np.min(ptimes))
        tt_session.add(tt_entry)
        tt_session.commit() # Probably faster to do the commit outside of loop but oh well
    tt_session.close()
    return inv

def build_tt_tables_local_directory(dirname=None,project_folder=None,channel_codes=['EH','BH','HH'],db=None,maxdist=800.,source_depth=5.):
    """ 
    """
    # Create a connection to an sqlalchemy database
    tt_engine=create_engine(db,echo=False)
    tt_stations_1D.BaseTT1D.metadata.create_all(tt_engine)
    TTSession=sessionmaker(bind=tt_engine)
    tt_session=TTSession()
    inv = Inventory()
    dir1a = glob.glob(project_folder+'/'+dirname+'/*xml')
    for file1 in dir1a:
        inv1a = read_inventory(file1)
        inv.networks.extend(inv1a)
    for net in inv:
        network=net.code
        for sta in net:
            loccodes=[]
            for ch in sta:
                for cc in channel_codes:
                  if re.match(cc,ch.code):
                    if not ch.location_code in loccodes:
                      loccodes.append(ch.location_code)
            for loc in loccodes:
                print(sta.code,network,loc,sta.latitude,sta.longitude,sta.elevation)
                station=tt_stations_1D.Station1D(sta.code,network,loc,sta.latitude,sta.longitude,sta.elevation)
                tt_session.add(station)
            tt_session.commit()

    # Now we have to build our traveltime lookup tables
    # We will use IASP91 here but obspy.taup does let you build your own model
    velmod=taup.TauPyModel(model='iasp91')
    delta_distance=1. # km for spacing tt calculations  
    distance_km=np.arange(0,maxdist+delta_distance,delta_distance)
    for d_km in distance_km:
        d_deg=geodetics.kilometer2degrees(d_km)
        ptimes=[]
        stimes=[]
        p_arrivals=velmod.get_travel_times(source_depth_in_km=source_depth,
          distance_in_degree=d_deg,phase_list=['P','p'])
        for p in p_arrivals:
            ptimes.append(p.time)
        s_arrivals=velmod.get_travel_times(source_depth_in_km=source_depth,
            distance_in_degree=d_deg,phase_list=['S','s'])
        for s in s_arrivals:
            stimes.append(s.time)
        tt_entry=tt_stations_1D.TTtable1D(d_km,d_deg,np.min(ptimes),np.min(stimes),np.min(stimes)-np.min(ptimes))
        tt_session.add(tt_entry)
        tt_session.commit() # Probably faster to do the commit outside of loop but oh well
    tt_session.close()
    return inv


    
def build_tt_tables_local_directory_ant(dirname=None,project_folder=None,channel_codes=['EH','BH','HH'],db=None,maxdist=800.,source_depth=5.):
    """ 
    """
    # Create a connection to an sqlalchemy database
    tt_engine=create_engine(db,echo=False)
    tt_stations_1D.BaseTT1D.metadata.create_all(tt_engine)
    TTSession=sessionmaker(bind=tt_engine)
    tt_session=TTSession()
    inv = Inventory()
    dir1a = glob.glob(project_folder+'/'+dirname+'/*xml')
    m = Basemap(projection='spstere',boundinglat=-60,lon_0=180,resolution='i')
    
    for file1 in dir1a:
        inv1a = read_inventory(file1)
        inv.networks.extend(inv1a)
    for net in inv:
        network=net.code
        for sta in net:
            loccodes=[]
            for ch in sta:
                for cc in channel_codes:
                  if re.match(cc,ch.code):
                    if not ch.location_code in loccodes:
                      loccodes.append(ch.location_code)
            for loc in loccodes:
                print(sta.code,network,loc,sta.latitude,sta.longitude,sta.elevation)
                x,y = m(sta.longitude,sta.latitude)

                station=tt_stations_1D.Station1D(sta.code,network,loc,y,x,sta.elevation)
                tt_session.add(station)
            tt_session.commit()

    # Now we have to build our traveltime lookup tables
    # We will use IASP91 here but obspy.taup does let you build your own model
    velmod=taup.TauPyModel(model='iasp91')
    delta_distance=1. # km for spacing tt calculations  
    distance_km=np.arange(0,maxdist+delta_distance,delta_distance)
    for d_km in distance_km:
        d_deg=geodetics.kilometer2degrees(d_km)
        ptimes=[]
        stimes=[]
        p_arrivals=velmod.get_travel_times(source_depth_in_km=source_depth,
          distance_in_degree=d_deg,phase_list=['P','p'])
        for p in p_arrivals:
            ptimes.append(p.time)
        s_arrivals=velmod.get_travel_times(source_depth_in_km=source_depth,
            distance_in_degree=d_deg,phase_list=['S','s'])
        for s in s_arrivals:
            stimes.append(s.time)
        tt_entry=tt_stations_1D.TTtable1D(d_km,d_deg,np.min(ptimes),np.min(stimes),np.min(stimes)-np.min(ptimes))
        tt_session.add(tt_entry)
        tt_session.commit() # Probably faster to do the commit outside of loop but oh well
    tt_session.close()
    return inv
    
def fb_pick(dbengine=None,picker=None,fileinput=None):
    fdir = []
    engine_assoc=dbengine
    with open(fileinput) as f:
        for line in f:
            tmp = line.split()
            fdir.append([tmp[0], tmp[1], tmp[2]])
        nsta = len(fdir)

        for i in range(nsta):
            Session=sessionmaker(bind=engine_assoc)
            dbsession=Session()
            st = Stream()
            st += read(fdir[i][0])
            st += read(fdir[i][1])
            st += read(fdir[i][2])
            st.merge()
            #print(st)
            for tr in st:
                if isinstance(tr.data, np.ma.masked_array):
                    tr.data = tr.data.filled()
            st.detrend(type='linear')
            for tr in st:
                print(tr)
                scnl,picks,polarity,snr,uncert=picker.picks(tr)
                t_create=datetime.utcnow()
                for i in range(len(picks)):
                    new_pick=tables1D.Pick(scnl,picks[i].datetime,polarity[i],snr[i],uncert[i],t_create)
                    dbsession.add(new_pick)

def gpd_pick_add(dbsession=None,fileinput=None,inventory=None):
    filepath = fileinput
    with open(filepath) as fp:
        line = fp.readline()
        cnt = 1
        while line:
            try:
                line = fp.readline()
                #print(line)
                cnt += 1
                if len(line.split())>4:
                    sta1 = line.split()[1]
                    chan1 = line.split()[2]
                    #print(sta1,chan1)
                    #scnl.station = sta1
                    net1 = line.split()[0]
                    scnl = SCNL([sta1,chan1,net1])
                    #print(scnl.channel)
                    type1 = line.split()[3]
                    scnl.phase = type1
                    #print(scnl.phase)
                    time1 = UTCDateTime(line.split()[4]).datetime
                else:
                    sta1 = line.split()[0]
                    chan1 = line.split()[1]
                    #print(sta1,chan1)
                    #scnl.station = sta1
                    #net1 = line.split()[0]
                    try:
                        net1 = inventory.select(station=sta1)[0].code
                    except:
                        net1 = 'OK'
                        pass
                    scnl = SCNL([sta1,chan1,net1])
                    #print(scnl.channel)
                    type1 = line.split()[2]
                    scnl.phase = type1
                    #print(scnl.phase)
                    time1 = UTCDateTime(line.split()[3]).datetime
                t_create=datetime.utcnow()
                new_pick=tables1D.Pick(scnl,time1,'',10,0.1,t_create)
                #tables1D.Pick.phase=type1
                dbsession.add(new_pick) # Add pick i to the database
                dbsession.commit() #
            except:
                pass

# def gpd_pick_add(dbsession=None,fileinput=None):
#   filepath = fileinput
#   with open(filepath) as fp:
#     line = fp.readline()
#     cnt = 1
#     while line:
#       try:
#         print("Line {}: {}".format(cnt, line.strip()))
#         line = fp.readline()
#         cnt += 1
#         sta1 = line.split()[1]
#         chan1 = line.split()[2]
#         #print(sta1,chan1)
#         #scnl.station = sta1
#         net1 = line.split()[0]
#         scnl = SCNL([sta1,chan1,'OK'])
#         #print(scnl.channel)
#         type1 = line.split()[3]
#         scnl.phase = type1
#         time1 = UTCDateTime(line.split()[4]).datetime
#         t_create=datetime.utcnow()
           
#         new_pick=tables1D.Pick(scnl,time1,'',10,0.1,t_create)
#         dbsession.add(new_pick) # Add pick i to the database
#         dbsession.commit() #
#       except:
#         pass
                
def get_chan1(stationfile):
    if len(list(filter(None, stationfile.split('/')[-1].split('.'))))==5:
        comp = list(filter(None, stationfile.split('/')[-1].split('.')))[3][2]
    else:
        comp = list(filter(None, stationfile.split('/')[-1].split('.')))[2][2]
    return comp

def get_chan3(stationfile):
    if len(list(filter(None, stationfile.split('/')[-1].split('.'))))==5:
        comp3 = list(filter(None, stationfile.split('/')[-1].split('.')))[3][0:3]
    else:
        comp3 = list(filter(None, stationfile.split('/')[-1].split('.')))[2][0:3]
    return comp3
           
def detection_continuous(dirname=None, project_folder=None, project_code=None, local=True, machine=True,single_date=None, latitude=None, longitude=None, max_radius=None):
#    starting = UTCDateTime(single_date.strftime("%Y")+'-'+single_date.strftime("%m")+'-'+single_date.strftime("%d")+'T00:00:00.0')
#    stopping = starting + 86430
    starting = UTCDateTime(single_date.strftime("%Y")+'-'+single_date.strftime("%m")+'-'+single_date.strftime("%d")+'T00:00:00.0')
    stopping = starting + 86430
    dir1 = project_folder+'/'+dirname
    #print(single_date.strftime("%Y%m%d"))
    #print(dir1+'/1dassociator_'+project_code+'.db')
    if os.path.exists(dir1+'/1dassociator_'+project_code+'.db'):
        os.remove(dir1+'/1dassociator_'+project_code+'.db')
    db_assoc='sqlite:///'+dir1+'/1dassociator_'+project_code+'.db'    
#    if os.path.exists(dir1+'/tt_ex_1D_'+project_code+'.db'):
#        os.remove(dir1+'/tt_ex_1D_'+project_code+'.db')
#    db_tt='sqlite:///'+dir1+'/tt_ex_1D_'+project_code+'.db' # Traveltime database44.448,longitude=-115.136
#    print(db_tt)
#    if local:
#        inventory = build_tt_tables_local_directory(dirname=dirname,project_folder=project_folder,channel_codes=['EH','BH','HH'],db=db_tt,maxdist=maxdist,source_depth=5.)
#    else:
#        inventory = build_tt_tables(lat1=latitude,long1=longitude,maxrad=max_radius,starting=starting, stopping=stopping, channel_codes=['EH','BH','HH'],db=db_tt,maxdist=maxdist,source_depth=5.)
    engine_assoc=create_engine(db_assoc, echo=False)
    tables1D.Base.metadata.create_all(engine_assoc)
    Session=sessionmaker(bind=engine_assoc)
    session=Session()
    filelist = glob.glob(dir1+'/*mseed') or glob.glob(dir1+'/*SAC')
    stations = set()
    for file1 in filelist:
        station = file1.split('.')[1]
        net = file1.split('.')[0].split('/')[-1]
        netsta = net+'.'+station
        print(file1.split('.')[1])
        stations.add(netsta)
    #### create infile
    day_strings = []
    for stationin in stations:
        station3 = glob.glob(dir1+'/*'+stationin+'.*mseed') or glob.glob(dir1+'/*'+stationin+'.*SAC')
        station3a = [None,None,None]
        if len(station3)>3:
            #print(station3)
            ind1 = np.empty((6,1))
            ind1[:] = np.nan
            for idxs, station1 in enumerate(station3):
                if get_chan3(station1) == 'HHZ':
                    ind1[idxs] = 2
                elif get_chan3(station1) == 'HHN' or get_chan3(station1) == 'HH1':
                    ind1[idxs] = 0
                elif get_chan3(station1) == 'HHE' or get_chan3(station1) == 'HH2':
                    ind1[idxs] = 1
                #print(idxs)
                #if ind1:
                #    station3a[ind1] = station1
            #ind2 = np.argwhere(~np.isnan(ind1))[:,0]
            for idxsa, ind2a in enumerate(ind1):
                if ~np.isnan(ind2a[0]):
                    #print(ind2a)
                    #print(station3a)
                    station3a[int(ind2a[0])] = station3[idxsa]
        else:
            for station1 in station3:
                if get_chan1(station1)  == 'Z':
                    ind1 = 2
                elif get_chan1(station1)  == 'N' or get_chan1(station1) == '1':
                    ind1 = 0
                elif get_chan1(station1)  == 'E' or get_chan1(station1) == '2':
                    ind1 = 1
                #print(ind1)
                station3a[ind1] = station1
        if any(elem is None for elem in station3a):
            continue
        day_strings.append((station3a[0]+' '+station3a[1]+' '+station3a[2]))
        
    day_string = "\n".join(day_strings)
    
    with open(dir1+'/dayfile.in', "w") as open_file:
        open_file.write(day_string)
    infile = dir1+'/dayfile.in'
    outfile = dir1+'/gpd_picks.out'
    #gpd_predict.py -V -P -I infile -O outflie
    #os.system("gpd_predict.py -V -P -I %s -O %s")%(infile, outfile)
    #gpd_predict(inputfile=infile,outputfile=outfile)
    fileinassociate = outfile
    
    if local:
        inv = Inventory()
        dir1a = glob.glob(project_folder+'/'+dirname+'/*xml')
        for file1 in dir1a:
            inv1a = read_inventory(file1)
            inv.networks.extend(inv1a)
    else:
        fdsnclient=Client()
        inv=fdsnclient.get_stations(starttime=starting,endtime=stopping,latitude=latitude,longitude=longitude,maxradius=max_radius,channel='*HZ',level='channel')
    if machine:
        fullpath1 = pathgpd+'/gpd_predict.py'
        os.system(fullpath1+" -V -P -I %s -O %s -F %s" % (infile, outfile, pathgpd))
        gpd_pick_add(dbsession=session,fileinput=fileinassociate,inventory=inv)
    else:
        picker = fbpicker.FBPicker(t_long = 5, freqmin = 1, mode = 'rms', t_ma = 20, nsigma = 7, t_up = 0.7, nr_len = 2, nr_coeff = 2, pol_len = 10, pol_coeff = 10, uncert_coeff = 3)
        fb_pick(dbengine=engine_assoc,picker=picker,fileinput=infile)
#    assocXX=assoc1D.LocalAssociator(db_assoc, db_tt, max_km = maxkm, aggregation = 1, aggr_norm = 'L2', cutoff_outlier = 10, assoc_ot_uncert = 3, nsta_declare = 4, loc_uncert_thresh = 0.5)
#    print("aggregate")
#    t0=datetime.utcnow()
#      # Identify candidate events (Pick Aggregation)
#    assocXX.id_candidate_events()
#    t1=datetime.utcnow()
#    print('Took '+str(t1-t0))
#    print("associate")
#      # Associate events
#    assocXX.associate_candidates()
#    t2=datetime.utcnow()
#    print('Took '+str(t2-t1))
#      # Add singles stations to events
#    assocXX.single_phase()

def association_continuous(dirname=None, project_folder=None, project_code=None, maxdist = None, maxkm=None, single_date=None, local=True, latitude=None, longitude=None, max_radius=None):
    starting = UTCDateTime(single_date.strftime("%Y")+'-'+single_date.strftime("%m")+'-'+single_date.strftime("%d")+'T00:00:00.0')
    stopping = starting + 86430

    dir1 = project_folder+'/'+dirname
    print(single_date.strftime("%Y%m%d"))
    #print(dir1+'/1dassociator_'+project_code+'.db')
#    if os.path.exists(dir1+'/1dassociator_'+project_code+'.db'):
#        os.remove(dir1+'/1dassociator_'+project_code+'.db')
#    db_assoc='sqlite:///'+dir1+'/1dassociator_'+project_code+'.db'    
    if os.path.exists(dir1+'/tt_ex_1D_'+project_code+'.db'):
        os.remove(dir1+'/tt_ex_1D_'+project_code+'.db')
    db_tt='sqlite:///'+dir1+'/tt_ex_1D_'+project_code+'.db' # Traveltime database44.448,longitude=-115.136
    print(db_tt)
    if local:
        inventory = build_tt_tables_local_directory(dirname=dirname,project_folder=project_folder,channel_codes=['EH','BH','HH'],db=db_tt,maxdist=maxdist,source_depth=5.)
    else:
        inventory = build_tt_tables(lat1=latitude,long1=longitude,maxrad=max_radius,starting=starting, stopping=stopping, channel_codes=['EH','BH','HH'],db=db_tt,maxdist=maxdist,source_depth=5.)
    inventory.write(dir1+'/dailyinventory.xml',format="STATIONXML")
#    engine_assoc=create_engine(db_assoc, echo=False)
#    tables1D.Base.metadata.create_all(engine_assoc)
#    Session=sessionmaker(bind=engine_assoc)
#    session=Session()
#    filelist = glob.glob(dir1+'/*mseed') or glob.glob(dir1+'/*SAC')
#    stations = set()
#    for file1 in filelist:
#        station = file1.split('.')[1]
#        net = file1.split('.')[0].split('/')[-1]
#        netsta = net+'.'+station
#        print(file1.split('.')[1])
#        stations.add(netsta)
#    #### create infile
#    day_strings = []
#    for stationin in stations:
#        station3 = glob.glob(dir1+'/*'+stationin+'.*mseed') or glob.glob(dir1+'/*'+stationin+'.*SAC')
#        station3a = [None,None,None]
#        if len(station3)>3:
#            #print(station3)
#            ind1 = np.empty((6,1))
#            ind1[:] = np.nan
#            for idxs, station1 in enumerate(station3):
#                if list(filter(None, station1.split('/')[-1].split('.')))[2][0:3] == 'HHZ':
#                    ind1[idxs] = 2
#                elif list(filter(None, station1.split('/')[-1].split('.')))[2][0:3] == 'HHN' or list(filter(None, station1.split('/')[-1].split('.')))[2][0:3] == 'HH1':
#                    ind1[idxs] = 0
#                elif list(filter(None, station1.split('/')[-1].split('.')))[2][0:3] == 'HHE' or list(filter(None, station1.split('/')[-1].split('.')))[2][0:3] == 'HH2':
#                    ind1[idxs] = 1
#                #print(idxs)
#                #if ind1:
#                #    station3a[ind1] = station1
#            #ind2 = np.argwhere(~np.isnan(ind1))[:,0]
#            for idxsa, ind2a in enumerate(ind1):
#                if ~np.isnan(ind2a[0]):
#                    #print(ind2a)
#                    #print(station3a)
#                    station3a[int(ind2a[0])] = station3[idxsa]
#        else:
#            for station1 in station3:
#                if list(filter(None, station1.split('/')[-1].split('.')))[2][2]  == 'Z':
#                    ind1 = 2
#                elif list(filter(None, station1.split('/')[-1].split('.')))[2][2]  == 'N' or list(filter(None, station1.split('/')[-1].split('.')))[2][2] == '1':
#                    ind1 = 0
#                elif list(filter(None, station1.split('/')[-1].split('.')))[2][2]  == 'E' or list(filter(None, station1.split('/')[-1].split('.')))[2][2] == '2':
#                    ind1 = 1
#                #print(ind1)
#                station3a[ind1] = station1
#        if any(elem is None for elem in station3a):
#            continue
#        day_strings.append((station3a[0]+' '+station3a[1]+' '+station3a[2]))
#        
#    day_string = "\n".join(day_strings)
#    
#    with open(dir1+'/dayfile.in', "w") as open_file:
#        open_file.write(day_string)
#    infile = dir1+'/dayfile.in'
#    outfile = dir1+'/gpd_picks.out'
#    #gpd_predict.py -V -P -I infile -O outflie
#    #os.system("gpd_predict.py -V -P -I %s -O %s")%(infile, outfile)
#    os.system("/home/jwalter/seis/generalized-phase-detection/gpd_predict.py -V -P -I %s -O %s" % (infile, outfile))
#    #gpd_predict(inputfile=infile,outputfile=outfile)
#    fileinassociate = outfile
#    gpd_pick_add(dbsession=session,fileinput=fileinassociate,inventory=inventory)
    db_assoc='sqlite:///'+dir1+'/1dassociator_'+project_code+'.db'
    assocXX=assoc1D.LocalAssociator(db_assoc, db_tt, max_km = maxkm, aggregation = 1, aggr_norm = 'L2', cutoff_outlier = 10, assoc_ot_uncert = 3, nsta_declare = 4, loc_uncert_thresh = 0.2)
    print("aggregate")
    t0=datetime.utcnow()
      # Identify candidate events (Pick Aggregation)
    assocXX.id_candidate_events()
    t1=datetime.utcnow()
    print('Took '+str(t1-t0))
    print("associate")
      # Associate events
    assocXX.associate_candidates()
    t2=datetime.utcnow()
    print('Took '+str(t2-t1))
      # Add singles stations to events
    try:
        assocXX.single_phase()
    except:
        pass
    
    

def detection_assocation_event(dirname=None, project_folder=None, project_code=None, maxdist=None, maxkm=None, starting=None, stopping=None):
    # starting = UTCDateTime(single_date.strftime("%Y")+'-'+single_date.strftime("%m")+'-'+single_date.strftime("%d")+'T00:00:00.0')
    # stopping = starting + 86430

    dir1 = project_folder+'/'+dirname
    #print(single_date.strftime("%Y%m%d"))
    if os.path.exists(dir1+'/1dassociator_'+project_code+'.db'):
        os.remove(dir1+'/1dassociator_'+project_code+'.db')
    db_assoc='sqlite:///'+dir1+'/1dassociator_'+project_code+'.db'    
    if os.path.exists(dir1+'/tt_ex_1D_'+project_code+'.db'):
        os.remove(dir1+'/tt_ex_1D_'+project_code+'.db')
    db_tt='sqlite:///'+dir1+'/tt_ex_1D_'+project_code+'.db' # Traveltime database44.448,longitude=-115.136
    inventory = build_tt_tables_local_directory(dirname=dirname,project_folder=project_folder,channel_codes=['EH','BH','HH'],db=db_tt,maxdist=maxdist,source_depth=5.)

    engine_assoc=create_engine(db_assoc, echo=False)
    tables1D.Base.metadata.create_all(engine_assoc)
    Session=sessionmaker(bind=engine_assoc)
    session=Session()
    filelist = glob.glob(dir1+'/*mseed')
    stations = set()
    for file1 in filelist:
        station = file1.split('.')[1]
        net = file1.split('.')[0].split('/')[-1]
        netsta = net+'.'+station
        print(file1.split('.')[1])
        stations.add(netsta)
    #### create infile
    day_strings = []
    for stationin in stations:
        station3 = glob.glob(dir1+'/*'+stationin+'*mseed')
        station3a = [None,None,None]
        if len(station3)>3:
            print(station3)
            ind1 = np.empty((6,1))
            ind1[:] = np.nan
            for idxs, station1 in enumerate(station3):
                if station1.split('.')[2] == 'HHZ':
                    ind1[idxs] = 2
                elif station1.split('.')[2] == 'HHN' or station1.split('.')[2] == 'HH1':
                    ind1[idxs] = 0
                elif station1.split('.')[2] == 'HHE' or station1.split('.')[2] == 'HH2':
                    ind1[idxs] = 1
                #print(idxs)
                #if ind1:
                #    station3a[ind1] = station1
            #ind2 = np.argwhere(~np.isnan(ind1))[:,0]
            for idxsa, ind2a in enumerate(ind1):
                if ~np.isnan(ind2a[0]):
                    #print(ind2a)
                    #print(station3a)
                    station3a[int(ind2a[0])] = station3[idxsa]
        else:
            for station1 in station3:
                if station1.split('.')[3].split('__')[0][-1]  == 'Z':
                    ind1 = 2
                elif station1.split('.')[3].split('__')[0][-1]  == 'N' or station1.split('.')[3].split('__')[0][-1] == '1':
                    ind1 = 0
                elif station1.split('.')[3].split('__')[0][-1]  == 'E' or station1.split('.')[3].split('__')[0][-1] == '2':
                    ind1 = 1
                #print(ind1)
                station3a[ind1] = station1
        if any(elem is None for elem in station3a):
            continue
        day_strings.append((station3a[0]+' '+station3a[1]+' '+station3a[2]))
        
    day_string = "\n".join(day_strings)
    
    with open(dir1+'/dayfile.in', "w") as open_file:
        open_file.write(day_string)
    infile = dir1+'/dayfile.in'
    outfile = dir1+'/gpd_picks.out'
    #gpd_predict.py -V -P -I infile -O outflie
    #os.system("gpd_predict.py -V -P -I %s -O %s")%(infile, outfile)
    fullpath1 = pathgpd+'/gpd_predict.py'

    os.system(fullpath1+" -V -P -I %s -O %s -F %s" % (infile, outfile, pathgpd))
    #gpd_predict(inputfile=infile,outputfile=outfile)
    fileinassociate = outfile
    print(outfile)
    gpd_pick_add(dbsession=session,fileinput=fileinassociate)
    print(str(maxkm)+' max km')
    assocXX=assoc1Dice1.LocalAssociator(db_assoc, db_tt, max_km = maxkm, aggregation = 1, aggr_norm = 'L2', cutoff_outlier = 15, assoc_ot_uncert = 10, nsta_declare = 3, loc_uncert_thresh = 0.5)
    print("aggregate")
    t0=datetime.utcnow()
      # Identify candidate events (Pick Aggregation)
    assocXX.id_candidate_events()
    t1=datetime.utcnow()
    print('Took '+str(t1-t0))
    print("associate")
      # Associate events
    assocXX.associate_candidates()
    t2=datetime.utcnow()
    print('Took '+str(t2-t1))
      # Add singles stations to events
    assocXX.single_phase()

def create_connection(db_file):
    """ create a database connection to the SQLite database
        specified by the db_file
    :param db_file: database file
    :return: Connection object or None
    """
    try:
        conn = sqlite3.connect(db_file)
        return conn
    except Error as e:
        print(e)
 
    return None



def hypo_station(project_folder=None, project_code=None):
    hypo71_string_sta = ""
    station_strings = []
    f1 = open(project_folder+'/'+'sta','w')
    #f2 = open(project_folder+'/'+'station.dat', 'w')
    #for stas in temp:
    files = sorted(glob.glob(project_folder+'/*/tt*'+project_code+'.db')) or glob.glob(project_folder+'/tt*'+project_code+'.db')
    #print(files)
    stas1 = pd.DataFrame()
    for dfilesta in files: 
        conn1 = create_connection(dfilesta)
        with conn1:
            cur1 = conn1.cursor()
            cur1.execute("SELECT * FROM stations")
         
            rows = cur1.fetchall()
            
            for row in rows:
                #print(row[0],row[1])
                #(row[0])
                df4 = pd.DataFrame()
                df4 = pd.DataFrame({'station': row[1], 'net':row[2],'latitude':row[4],'longitude':row[5],'elevation':row[6]}, index=[0])
                stas1=stas1.append(df4)
    stas1 = stas1.drop_duplicates()
    stas1 = stas1.reset_index(drop=True)
    print(stas1)
    for idx1 in stas1.index:
        stas = stas1.iloc[idx1]
        print(stas)
#                temp = stas1[stas1['station'].str.contains(sta_used)]
#                stas = temp.iloc[0]
        

    
        if len(stas['station'])>4:
            sta = stas['station'][1:]
        else:
            sta = stas['station']
        lon = stas['longitude']
        lon_deg = int(abs(lon))
        lon_min = (abs(lon) - abs(lon_deg)) * 60.
        lat = stas['latitude']
        lat_deg = int(abs(lat))
        lat_min = (abs(lat) - abs(lat_deg)) * 60.
        hem_NS = 'N'
        hem_EW = 'E'
        if lat < 0:
            hem_NS = 'S'
        if lon < 0:
            hem_EW = 'W'
        # hypo 71 format uses elevation in meters not kilometers
        ele = stas['elevation']
        hypo71_string_sta += fmt % (sta, lat_deg, lat_min, hem_NS,
                                lon_deg, lon_min, hem_EW, ele)
        station_strings.append("%s %.6f %.6f %i" % (sta, stas['latitude'], stas['longitude'], stas['elevation']))
                
                
                #print(hypo71_string_sta)
    station_string = "\n".join(station_strings)
    with open(project_folder+'/'+'station.dat', "w") as open_file:
        open_file.write(station_string)
    f1.write(str(hypo71_string_sta))
    f1.close()




 
def select_all_associated(conn,f0):
    """
    Query all rows in the associated table
    :param conn: the Connection object
    :return:
    """
    cur1 = conn.cursor()
    cur1.execute("SELECT * FROM associated")
    stalistall = set()
    rows = cur1.fetchall()
    dfs1 = pd.DataFrame()
    cat1 = Catalog()
    for row in rows:
        #print(row)
        #(row[0])
        df4 = pd.DataFrame()
        df4 = pd.DataFrame({'Time': row[1], 'Lat':row[3],'Long':row[4]}, index=[0])
        dfs1=dfs1.append(df4)
        origin = Origin()
        origin.latitude = row[3]
        origin.longitude = row[4]
        origin.depth = row[5]
        origin.time = row[1]
        origin.arrivals = []
        strday = row[1][0:4]+row[1][5:7]+row[1][8:10]
        cur1.execute('SELECT * FROM picks_modified WHERE assoc_id IN (?)',[int(row[0])])
        picks1a = sorted(cur1.fetchall())
        stas = []
        event = Event()
        for pick1 in picks1a:
            
            #print(pick1)
            stream_id = WaveformStreamID(network_code=pick1[3], station_code=pick1[1], location_code="", channel_code=pick1[2])
            p = Pick()
            p.time = pick1[5]
            p.phase_hint = pick1[6]
            p.waveform_id = stream_id
            p.evaluation_mode = 'automatic'
            pres_id = 'smi:local/Pick/'+strday+'/'+str(pick1[0])
            #res_id = ResourceIdentifier(prefix='Pick')
            #res_id.convert_id_to_quakeml_uri(authority_id='obspy.org')
            p.resource_id = ResourceIdentifier(id=pres_id)
            #print(p)
            
            a = Arrival()
            #a.time = pick1[5]
            a.phase = pick1[6]
            a.pick_id = p.resource_id
            ares_id = 'smi:local/Arrival/'+strday+'/'+str(pick1[0])
            #res_id = ResourceIdentifier(prefix='Pick')
            #res_id.convert_id_to_quakeml_uri(authority_id='obspy.org')
            a.resource_id = ResourceIdentifier(id=ares_id)
            #print(a)
            
            #origin.picks.append(p)
            sta1 = pick1[1]
            stas.append(sta1)
            stalistall.add(sta1)
            origin.arrivals.append(a)
            event.picks.append(p)
        event.origins.append(origin)
        cat1.append(event)
            #print(stalistall)
        stalist = list(set(stas))
        for states in stalist:
            hypo71_string = ""
            numP = -9
            numS = -9
            #print(states)
            for num, line in enumerate(picks1a):
                if states in line and 'P' in line:
                    numP = num
                if states in line and 'S' in line:
                    numS = num
#            if numP > -1 and numS < -1:
#                #print('just P'+str(numP))
#            if numP < -1 and numS > -1:
#                #print('just S')
#            if numP > -1 and numS > -1:
#                print('both'+str(numP)+' '+str(numS))
            if len(states)>4:
                sta = states[1:]
            else:
                sta = states
            if numP > -1:
                pick = picks1a[numP]
            
            
            
                t = UTCDateTime(pick[5])
                hundredth = int(round(t.microsecond / 1e4))
                if hundredth == 100:
                    t_p = t + 1
                    hundredth = 0
                else:
                    t_p = t
                date = t_p.strftime("%y%m%d%H%M%S") + ".%02d" % hundredth
                onset = 'I'
                polarity = '?'
                weight = 1
                #print(sta,onset,polarity,weight,date)
                hypo71_string += fmtP % (sta, onset, polarity, weight, date)
                #f0.write(str(hypo71_string))
                
                #print(hypo71_string)
                if numP > -1 and numS > -1:
                    pick = picks1a[numS]
                    #t = UTCDateTime(pick[5])
                    t2 = UTCDateTime(pick[5])
                    # if the S time's absolute minute is higher than that of the
                    # P pick, we have to add 60 to the S second count for the
                    # hypo 2000 output file
                    # +60 %60 is necessary if t.min = 57, t2.min = 2 e.g.
                    mindiff = (t2.minute - t.minute + 60) % 60
                    abs_sec = t2.second + (mindiff * 60)
                    hundredth = int(round(t2.microsecond / 1e4))
                    if hundredth == 100:
                        abs_sec += 1
                        hundredth = 0
                    date2 = "%s.%02d" % (abs_sec, hundredth)
                    hundredth = int(round(t.microsecond / 1e4))
                    if hundredth == 100:
                        t_p = t + 1
                        hundredth = 0
                    else:
                        t_p = t
                    date = t_p.strftime("%y%m%d%H%M%S") + ".%02d" % hundredth
                    onset = 'I'
                    polarity = '?'
                    weight = 1
                    #print(sta,onset,polarity,weight,date)
                    hypo71_string += fmtS % (date2, onset, polarity,weight)
                    
                else:
                    hypo71_string += "\n"
                    #f0.write("\n")
                f0.write(str(hypo71_string))
            #cur1.execute('SELECT * FROM picks WHERE id IN (?)',[int(pick[0])])
            #extrapick = cur1.fetchall()
        f0.write("\n")
      
            
    return dfs1, stalistall, cat1, f0

def combine_associated(project_folder=None, project_code=None):
    #files = sorted(glob.glob('/data/tx/ContWaveform/*/1dass*'++'.db'))
    #files = [f for f in os.listdir(dirdata) if os.path.isfile(os.path.join(dirdata, f))]
    #dir1 = project_folder+'/'+dirname
    
    hypo_station(project_folder, project_code)
    files = sorted(glob.glob(project_folder+'/*/1dass*'+project_code+'.db'))
    f0 = open(project_folder+'/pha_'+project_code,'w')
    dfs2 = pd.DataFrame()
    stalistall1 = []
    cat = Catalog()
    for dfile in files:   
        # create a database connection
        print(dfile)
        conn = create_connection(dfile)
        
        with conn:
            #print("1. Query task by priority:")
            #select_task_by_priority(conn,1)
         
            print('Day '+dfile[-6:-3])
            #try:
                
            dfs1,stalistall,cat1,f0 = select_all_associated(conn,f0)
            cat.extend(cat1)
            for stas1 in stalistall:
                if stas1 not in stalistall1:
                    stalistall1.append(stas1)
            dfs2 = dfs2.append(dfs1)
#            except:
#                pass
    f0.close()
    cat.write(project_folder+'/'+project_code+'_cat.xml',format="QUAKEML")
    return cat, dfs2





#
#      index0=int(round((picks[i]-self.tr.stats.starttime)/dt,0))
#      index=index0
#      
#      # roll forward index+=1
#      while True:
#        if index>=self.stats.npts-1-2:
#          break
#        elif (self.tr[index+1]-self.tr[index])*(self.tr[index+2]-self.tr[index+1])>0:
#          index+=1
#        else:
#          break
#      
#      # notice index+1, rolling stop one point before extreme, compare with std to avoid very small 
#      if self.tr[index+1] - self.tr[index0] > 0 and abs(self.tr[index+1] - self.tr[index0]) > self.picker.pol_coeff * np.std(self.tr[index0 - self.picker.pol_len: index0]):
#        polarity='C'
#      elif self.tr[index+1] - self.tr[index0] < 0 and abs(self.tr[index+1] - self.tr[index0]) > self.picker.pol_coeff * np.std(self.tr[index0 - self.picker.pol_len: index0]):
#        polarity='D'
#      else: 
#        polarity=''
def polarity(tr,pickP=None):
    dt=tr.stats.delta
    #t = np.arange(0, tr.stats.npts/tr.stats.sampling_rate, dt) 
    index0=int(round((pickP-tr.stats.starttime)/dt,0))
    index=index0
    pol_coeff = 5
    pol_len = 5
    polarity = 'undecidable'
    while True:
        if index>=tr.stats.npts-1-2:
            break
        elif (tr[index+1]-tr[index])*(tr[index+2]-tr[index+1])>0:
            index+=1
        else:
            break
        if tr[index+1] - tr[index0] > 0 and abs(tr[index+1] - tr[index0]) > pol_coeff * np.std(tr[index0 - pol_len: index0]):
            polarity='positive'
        elif tr[index+1] - tr[index0] < 0 and abs(tr[index+1] - tr[index0]) > pol_coeff * np.std(tr[index0 - pol_len: index0]):
            polarity='negative'
        else: 
            polarity='undecidable'
    return polarity
    
        
def magnitude_quakeml(cat=None, project_folder=None,plot_event=False):
#    PAZ_WA = {'poles': [-6.283 + 4.7124j, -6.283 - 4.7124j],
#          'zeros': [0 + 0j], 'gain': 1.0, 'sensitivity': 2080}
    paz_wa = {'sensitivity': 2080, 'zeros': [0j], 'gain': 1,'poles': [-6.2832 - 4.7124j, -6.2832 + 4.7124j]}
    
    print('Computing magnitudes')
#    
#    #inventory = client.get_stations(starttime=starttime, endtime=endtime, network="*", sta=tr.stats.station, loc="*", channel=tr.stats.channel,level="response")
    client = Client()
#    starttime = UTCDateTime(start)
#    endtime = UTCDateTime(end)
    
    fminbp = .1 # lower bandpass limit
    fmaxbp = 20 # upper bandpass limit
    ##import datetime
    #inva = client.get_stations(latitude=36.5,longitude=-98.9,minradius=0.0, maxradius=5,starttime=starttime,endtime=endtime,level="station")
    #invsta = client.get_stations(latitude=31.699,longitude=-104.053, maxradius=4,starttime=starttime,endtime=endtime,includeavailability=True,includerestricted=False,level="response")
#    if download_inventory:
#        invsta = client.get_stations(latitude=31.699,longitude=-104.053, maxradius=4,starttime=starttime,endtime=endtime,includeavailability=True,includerestricted=False,level="response")
#    else:
#        inv = Inventory()
#        dir1a = glob.glob(project_folder+'/*/*xml')
#        for file1 in dir1a:
#            inv1a = read_inventory(file1)
#            inv.networks.extend(inv1a)
    #cat3 = cat.filter("time > 2020-03-25T03:10","time < 2020-03-25T03:50")
    for event in cat:
        origin = event.origins[0]
        print(origin)
        event_lat = origin.latitude
        event_lon = origin.longitude
        strday = str(origin.time.year).zfill(2)+str(origin.time.month).zfill(2)+str(origin.time.day).zfill(2)
        #    strday = str(origin.time.year).zfill(2)+str(origin.time.month).zfill(2)+str(origin.time.day).zfill(2)
        strdaytime = strday+str(origin.time.hour).zfill(2)+str(origin.time.minute).zfill(2)[0]
        mags = []
        mags_iaspei = []

        st2 = Stream()
        for idx1, pick in enumerate(event.picks):
            if pick.phase_hint == 'S':
                ### make Amplitude
                try:
                    st3 = read(project_folder+'/'+strday+'*/'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'*mseed',debug_headers=True)
                except:
                    try:
                        st3 = read(project_folder+'/'+strday+'*/*.'+pick.waveform_id.station_code+'*SAC',debug_headers=True)
                    except:
                        st3 = read(project_folder+'/scratch/'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'*mseed',debug_headers=True)
#                    try:
#                        st3 = read(project_folder+'/'+strday+'*/*.'+pick.waveform_id.station_code+'*SAC',debug_headers=True)
#                    except:
#                        st3 = read(project_folder+'/'+strdaytime+'*/'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'*mseed',debug_headers=True)
                    pass

    #            pazs = glob.glob('/data/tx/ContWaveform/'+strday+'/SACPZ.'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'*[EN12]')
                #st = read(project_folder+'/'+strday+'*/'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'*[EN12]*.SAC',debug_headers=True)
                try:
                    st3.merge()
                    for tr in st3:
                        if isinstance(tr.data, np.ma.masked_array):
                            tr.data = tr.data.filled()
                    st = st3.select(channel='[HB]H[EN12]')
                    for tr in st3:
                        inventory_local = glob.glob(project_folder+'/'+strday+'*/'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'.xml')
                        if len(inventory_local)>0:
                            inv = read_inventory(inventory_local[0])
                        else:
                            if len(glob.glob(project_folder+'/'+strday+'*/'+pick.waveform_id.network_code+'.'+pick.waveform_id.station_code+'.xml'))>0:
                                inv0 = read_inventory(project_folder+'/'+strday+'*/dailyinventory.xml')
                                inv = inv0.select(network=pick.waveform_id.network_code, station=pick.waveform_id.station_code, time=origin.time)
                            else:
                                print('Getting response from DMC')
                                starttime = UTCDateTime(origin.time-10)
                                endtime = UTCDateTime(origin.time+10)
                                inv = client.get_stations(starttime=starttime, endtime=endtime, network="*", sta=tr.stats.station, loc="*", channel=tr.stats.channel,level="response")
                                #                    paz = [x for x in pazs if tr.stats.channel in x]
    #                    attach_paz(tr, paz[0])
                        #inv = client.get_stations(starttime=starttime, endtime=endtime, network="*", sta=tr.stats.station, loc="*", channel=tr.stats.channel,level="response")
                        tr.stats.network = inv[0].code
                        tr.stats.location = inv[0][0][0].location_code
                        pre_filt = (0.05, 0.06, 30.0, 35.0)
                        tr.trim(pick.time-20,pick.time+120)
                        
    
                        #tr.demean()
                        tr.detrend()
                        tr.remove_response(inventory=inv, output='VEL', pre_filt=pre_filt, zero_mean=True)
                        #tr.data = seis_sim(tr.data, tr.stats.sampling_rate,paz_remove=None, paz_simulate=paz_wa, water_level=10)
                        tr.simulate(paz_simulate=paz_wa, water_level=10)
                        
                        
                        #tr = tr.filter('bandpass', freqmin=fminbp, freqmax=fmaxbp, zerophase=True)
                    #st.trim(pick.time-5,pick.time+10)
                    tr1 = st3.select(channel='[EHB]HZ')[0]
    
                    sta_lat = inv[0][0].latitude 
                    sta_lon = inv[0][0].longitude
                    epi_dist, az, baz = gps2dist_azimuth(event_lat, event_lon, sta_lat, sta_lon)
                    epi_dist = epi_dist / 1000
                    tr1.stats.distance = gps2dist_azimuth(event_lat, event_lon, sta_lat, sta_lon)[0]
                    tr1.trim(pick.time-20,pick.time+60)
                    st2 += tr1
                    st.trim(pick.time-1,pick.time+5)
                    ampls = (max(abs(st[0].data)), max(abs(st[1].data)))
                    for idx2,ampl in enumerate(ampls):

                        amp = Amplitude()
                        res_id = 'smi:local/Amplitude/'+strday+'/'+str(10*idx2+idx1)
                        #res_id = ResourceIdentifier(prefix='Amplitude')
                        #res_id.convert_id_to_quakeml_uri(authority_id='obspy.org')
                        amp.resource_id = ResourceIdentifier(id=res_id)
                        amp.pick_id = pick.resource_id
                        amp.waveform_id = pick.waveform_id
                        amp.type = 'ML'
                        amp.generic_amplitude = ampl
                        amp.evaluation_mode = 'automatic'
    
                        if epi_dist < 60:
                            a = 0.018
                            b = 2.17
                        else:
                            a = 0.0038
                            b = 3.02
                        ml = np.log10(ampl * 1000) + a * epi_dist + b
                        
                        ml_iaspei = np.log10(ampl*1e6)+1.11*np.log10(epi_dist) + 0.00189*epi_dist - 2.09
                        print(ml, ml_iaspei)

                        if epi_dist < 160:
                            mags.append(ml)
                            mags_iaspei.append(ml_iaspei)
                            #### make StationMagnitude
                            stamag = StationMagnitude()
                            res_id = 'smi:local/StationMagnitude/'+strday+'/'+str(10*idx2+idx1)
                            #res_id = ResourceIdentifier(prefix='StationMagnitude')
                            #res_id.convert_id_to_quakeml_uri(authority_id='obspy.org')
                            stamag.resource_id = ResourceIdentifier(id=res_id)
                            stamag.origin_id = origin.resource_id
                            stamag.waveform_id = pick.waveform_id
                            stamag.mag = ml
                            stamag.station_magnitude_type = 'ML'
                            stamag.amplitude_id = amp.resource_id
                        ## add them to the event
                        event.station_magnitudes.append(stamag)
                        event.amplitudes.append(amp)
                except:
                    print('Something went wrong here')
                    pass
                
        for pick in event.picks:
            if pick.phase_hint == 'P':
                tr = st2.select(station=pick.waveform_id.station_code)
                try:
                    tr = tr[0]
                    pol = polarity(tr,pick.time)
                    pick.polarity = pol
                    print(pol)
                except:
                    pass
                
                
                
                
                
        netmag = np.median(mags_iaspei)
        try:
            m = Magnitude()
            m.mag = netmag
            m.mag_errors = {"uncertainty": np.std(mags_iaspei)}
            m.magnitude_type = 'ML'
            m.origin_id = origin.resource_id
            meth_id = 'smi:local/median'
            m.method_id = meth_id
            m.station_count = len(mags_iaspei)
            m_id = 'smi:local/Magnitude/'+strday+'/'+str(idx1)
            #m_id = ResourceIdentifier(prefix='StationMagnitude')
            #m_id.convert_id_to_quakeml_uri(authority_id='obspy.org')
            m.resource_id = m_id
            event.magnitudes.append(m)
            
            
            
            
            event.preferred_origin_id = event.origins[0].resource_id
            
            
            
            if plot_event:
                dir1a = glob.glob(project_folder+'/'+strday+'*')
                filename = dir1a[0]+'/'+strdaytime
                fig = plt.figure()
                st2.filter('highpass', freq=.1, zerophase=True)
                st2.plot(type='section', scale=2,plot_dx=100e3, recordlength=50,
                    time_down=True, linewidth=.25, grid_linewidth=.25, show=False, 
                    outfile=filename,fig=fig)
                plt.close()
        except:
            pass
    cat.write(project_folder+'/cat.xml',format="QUAKEML")
    return cat


def simple_cat_df(cat=None):
    times = []
    lats = []
    lons = []
    deps = []
    magnitudes = []
    magnitudestype = []
    resourceid = []
    for event in cat:
        if len(event.origins) != 0:
            times.append(event.origins[0].time.datetime)
            lats.append(event.origins[0].latitude)
            lons.append(event.origins[0].longitude)
            deps.append(event.origins[0].depth)
            if len(event.magnitudes) != 0:
                magnitudes.append(event.magnitudes[0].mag)
                magnitudestype.append(event.magnitudes[0].magnitude_type)
            else:
                magnitudes.append(np.nan)
                magnitudestype.append(np.nan)
            resourceid.append(event.resource_id)
    catdf1 = pd.DataFrame({'latitude':lats,'longitude':lons, 'depth':deps,'magnitude':magnitudes,'type':magnitudestype,'id':resourceid}, index = times)
    return catdf1

def catdf_narrowbounds(catdf=None,lat_a=None,lat_b=None,lon_a=None,lon_b=None):
    catdf = catdf[(catdf['latitude']>lat_a) & (catdf['latitude']<lat_b) & (catdf['longitude']>lon_a) & (catdf['longitude']<lon_b)]
    return catdf

#
#
def quakeml_to_hypodd(cat=None, project_folder=None, project_code=None):
    #catdf = simple_cat_df(cat)
    phase_dat_file = project_folder+'/'+project_code+'.pha'
    #for idx0, t0 in enumerate(catdf.index):
    #stations = []
    stations = set()

    event_strings = []            
    for idx1, event in enumerate(cat):
        evo = event.origins[0].time
        #evid = catdf['event_id'][idx1]
        #otime = UTCDateTime(evo)
        #try:
        
        origin = event.preferred_origin() or event.origins[0]
        mag1 = event.preferred_magnitude() or event.magnitudes[0]
        magpref = mag1.mag

        
        
        depth_error = 0
        longitude_error = 0
        latitude_error = 0

                    
        string = "# {year} {month} {day} {hour} {minute} " + \
            "{second:.6f} {latitude:.6f} {longitude:.6f} " + \
            "{depth:.4f} {magnitude:.6f} {horizontal_error:.6f} " + \
            "{depth_error:.6f} {travel_time_residual:.6f} {event_id}"
        event_string = string.format(year=origin.time.year,
                             month=origin.time.month,
                             day=origin.time.day,
                             hour=origin.time.hour,
                             minute=origin.time.minute,
                             # Seconds + microseconds
                             second=float(origin.time.second) +
                             (origin.time.microsecond / 1e6),
                             latitude=origin.latitude,
                             longitude=origin.longitude,
                             # QuakeML depth is in meters. Convert to km.
                             depth=origin.depth / 1000.0,
                             magnitude= magpref,
                             horizontal_error=max(
                                 [latitude_error,
                                  longitude_error]),
                             depth_error= depth_error,
                             travel_time_residual=0,
                             event_id=idx1)
        event_strings.append(event_string)
        for _i, arrv in enumerate(origin.arrivals):
            pick = arrv.pick_id.get_referred_object()
            stations.add(pick.waveform_id.station_code)
            #print(pick.polarity)
            # Only P and S phases currently supported by HypoDD.
            if pick.phase_hint.upper() != "P" and pick.phase_hint.upper() != "S":
                continue
            string = "{station_id} {travel_time:.6f} {weight:.2f} {phase}"
            travel_time = pick.time - origin.time
            # Simple check to assure no negative travel times are used.
            if travel_time < 0:
                msg = "Negative absolute travel time. " + \
                    "{phase} phase pick for event {event_id} at " + \
                    "station {station_id} will not be used."
                msg = msg.format(
                    phase=pick.phase_hint,
                    event_id=evid,
                    station_id=pick.waveform_id.station_code)
                print(msg)
                continue
            phase_weighting = lambda sta_id, ph_type, time, uncertainty: 1.0
            weight = phase_weighting(pick.waveform_id.station_code, pick.phase_hint.upper(),
                                     pick.time,
                                     arrv.time_residual)
            pick_string = string.format(
                station_id=pick.waveform_id.station_code,
                travel_time=travel_time,
                weight=weight,
                phase=pick.phase_hint.upper())
            event_strings.append(pick_string)
        event_string = "\n".join(event_strings)
#        except:
#            print('Some error occurred????', evo)
#            pass
        # Write the phase.dat file.
        with open(phase_dat_file, "w") as open_file:
            open_file.write(event_string)
            
def single_event_xml(catalog=None,project_folder=None):
    xmlspath = project_folder+'/xmls'
    if not os.path.exists(xmlspath):
        os.makedirs(xmlspath)
    for ev in catalog:
        filename = str(ev.resource_id).split('/')[-1] + ".xml"
        ev.write(xmlspath+'/'+filename, format="QUAKEML")            
            
    
#    station_dat_file = project_folder+'/'+'station.dat'
#    
#    #station_strings = []
#    #for key, value in self.stations.iteritems():
#    #    station_strings.append("%s %.6f %.6f %i" % (key, value["latitude"],
#    #        value["longitude"], value["elevation"]))
#    #station_string = "\n".join(station_strings)
#    #with open(station_dat_file, "w") as open_file:
#    #    open_file.write(station_string)
#    #self.log("Created station.dat input file.")
#    
#    
#    starttime = UTCDateTime("2010-01-01T00:00:00.000")
#    endtime = UTCDateTime("2022-01-01T00:00:00.000")
#    #line = [(-98.15, 35.88),(-98.05, 35.8)] # Cushing area
#    
#    client = Client('IRIS')
#    inva = client.get_stations(starttime=starttime, endtime=endtime,network="*", loc="*", channel="*",minlatitude=minlat, maxlatitude=maxlat,minlongitude=minlon, maxlongitude=maxlon,level="station")
#    station_strings = []
#    for sta in stations:
#        print(sta)
#        inva1 = inva.select(station=sta)
#        if len(inva1.networks) > 0:
#            station_strings.append("%s %.6f %.6f %i" % (sta, inva1[0][0].latitude, inva1[0][0].longitude, inva1[0][0].elevation))
#        #inva1[0][0].latitude
#    station_string = "\n".join(station_strings)
#    with open(station_dat_file, "w") as open_file:
#        open_file.write(station_string)




def plot_hypodd_catalog():    
    catdfr = pd.read_csv('/Users/jwalter/Dropbox/python/tx/hypoDD.relocpanfixed',delimiter=r"\s+")
    catdfr = catdfr.dropna()
    catdfr = catdfr.reset_index(drop=True)
    #rutc = np.zeros((len(catdfr.index),1))
    rutc = []
    for i in range(0,len(catdfr.index)):
        rutc.append(UTCDateTime(int(catdfr.iloc[i,10]),int(catdfr.iloc[i,11]),int(catdfr.iloc[i,12]),int(catdfr.iloc[i,13]),int(catdfr.iloc[i,14]),catdfr.iloc[i,15]))
    
    catdfr['rutc'] = rutc
    catdfr.sort_values(by=['rutc'], inplace=True)
    catdfr = catdfr.reset_index(drop=True)
    
    
    
    
    
    from mpl_toolkits.basemap import Basemap
    # 1. Draw the map background
    fig = plt.figure(figsize=(8, 8))
    m = Basemap(projection='lcc', resolution='h', 
                lat_0=31.66, lon_0=-104,
                width=1E6, height=.6E6)
    #m.shadedrelief()
    m.drawcoastlines(color='gray')
    m.drawcountries(color='gray')
    #m.drawcounties(color='gray')
    m.drawstates(color='gray')
    
    # 2. scatter city data, with color reflecting population
    # and size reflecting area
    m.scatter(catdfr.iloc[:,2].values,catdfr.iloc[:,1].values,s=catdfr.iloc[:,16].values**3*8,c=catdfr.index,marker='o',alpha=0.5,latlon=True)
    
    #m.scatter(catdfo.iloc[:,2].values,catdfo.iloc[:,1].values,s=catdfo.iloc[:,16].values**3*10,c=catdfo.index,marker='o',alpha=0.5,latlon=True)
    
    
    
    cbar = plt.colorbar()
    N_TICKS=8
    indexes = [catdfr['rutc'].iloc[i].strftime('%Y-%m-%d') for i in np.linspace(0,catdfr.shape[0]-1,N_TICKS).astype(int)] 
    
    #indexes = [catdfr.index[i].strftime('%Y-%m-%d') for i in np.linspace(0,catdfr.shape[0]-1,N_TICKS).astype(int)] 
    cbar.ax.set_yticklabels(indexes)

if __name__ == "__main__":
    easyQuake()





#start_date = date(2013, 12, 1)
#end_date = date(2013, 12, 2)
##end_date = date(2018, 1, 2)
#project_code = 'eq'
#project_folder = '/data/EasyQuake'
#for single_date in daterange(start_date, end_date):
#    print(single_date.strftime("%Y-%m-%d"))
#    dirname = single_date.strftime("%Y%m%d")
    #tempdate = glob.glob('/scratch/antarctica/TempWaveform/'+single_date.strftime("%Y%m%d")+'*/*SAC.bp')
    #os.path.basename(temp[0])
    #stanames = []
    #for f in tempdate:
        #print(f)
    #    stanames.append(os.path.basename(f).split(".")[1]+'.'+os.path.basename(f).split(".")[2])
    #stachan_uniq = set(stanames)



from .tt_stations_1D import *

def tt_km(session,d_km):
  """ tt_km(session,d_km)
  Look up travel time from tt_table for the closest distance to entered value stored in the 
  tt_table database. km_difference is the distance difference between the stored value
  returned in the lookup and the distance requested.
  
  session is the database connection
  
  Returns the TTtable_object,km_difference 
  """

  min=session.query(TTtable1D).filter(TTtable1D.d_km<=d_km).order_by(TTtable1D.d_km.desc()).first()
#   print min
  max=session.query(TTtable1D).filter(TTtable1D.d_km>=d_km).order_by(TTtable1D.d_km).first()
#   print max.d_km
  if abs(min.d_km-d_km) <= abs(max.d_km-d_km):
    return min,abs(min.d_km-d_km)
  else:
    return max,abs(max.d_km-d_km)

   
def tt_s_p(session,s_p):
  """ tt_s_p(session,s_p)
  Look up the distance for an S-P travel time observation and return the closest value
  stored in the travel-time table
  
  Returns the closest tt_object,s_p_difference
  """
#   print 's_p:',s_p
  min=session.query(TTtable1D).filter(TTtable1D.s_p<=s_p).order_by(TTtable1D.s_p.desc()).first()
#   print 'min.s_p:',min.s_p
  max=session.query(TTtable1D).filter(TTtable1D.s_p>=s_p).order_by(TTtable1D.s_p).first()
#   print 'max.s_p:',max.s_p
  if abs(min.s_p-s_p) <= abs(max.s_p-s_p):
    return min,abs(min.s_p-s_p)
  else:
    return max,abs(max.s_p-s_p)    
    



import numpy as np
from .util import *
from obspy.signal.filter import bandpass
from obspy.signal.invsim import cosine_taper as cosTaper

class Ratio():
  
  def __init__(self, trace, t_long, freqmin, cnr, perc_taper, mode = 'rms'):
    self.tr = trace
    self.t_long = t_long
    self.freqmin = freqmin
    self.statistics_mode = mode 
    self.cnr = cnr   
    self.perc_taper = perc_taper
    
  def _N_bands(self):
    """ Determine number of band n_bands in term of sampling rate.
    """
    df = self.tr.stats.sampling_rate
    Nyquist = df / 2.0
    n_bands = int(np.log2(Nyquist / 1.5 / self.freqmin)) + 1
    return n_bands

  def filter(self):
    """ Filter data for each band.
    """
    n_bands = self._N_bands()
    LEN = self.tr.stats.npts
    df = self.tr.stats.sampling_rate

    # create zeros 2D array for BF
    BF = np.zeros(shape = (n_bands,LEN))

    for j in range(n_bands):
      octave_high = (self.freqmin + self.freqmin * 2.0) / 2.0 * (2**j)
      octave_low = octave_high / 2.0
      BF[j] = bandpass(self.tr.data, octave_low, octave_high, df, corners = self.cnr, zerophase = False)
      BF[j] = cosTaper(LEN, self.perc_taper) * BF[j]

    return BF
    
  def _statistics(self):
    """ Calculate statistics for each band.
    """
    n_bands = self._N_bands()
    LEN = self.tr.stats.npts
    dt = self.tr.stats.delta
    npts_t_long = int(self.t_long / dt)
    
    # BF: band filtered data
    BF = self.filter()

    # E: the instantaneous energy
    E = np.power(BF, 2)

    # create zeros 2D array for rmsE, aveE and sigmaE
    rmsE = np.zeros(shape=(n_bands,LEN))
    aveE = np.zeros(shape=(n_bands,LEN))
    sigmaE = np.zeros(shape=(n_bands,LEN))
  
    # range start from 1,not 0, because sigmaE[i=0] calculation encontoured invalid value
    for i in np.arange(1,npts_t_long): 
      rmsE[:,i]=rms(E[:,:i],axis=1)
      aveE[:,i]=np.mean(E[:,:i],axis=1)
      sigmaE[:,i]=np.std(E[:,:i],axis=1)

    # call rolling_window to obtain the rmsE or sigmaE
    tmp=rolling_window(E[:,0:LEN-1],npts_t_long)
    if self.statistics_mode=='rms':
      rmsE[:,npts_t_long:LEN]=rms(tmp,axis=-1)
      rmsE=np.clip(rmsE,1.e-19,1.e19) # clip the sigmaE to avoid zeros values since FC encounters invalid value if denominator sigmaE is zero
    if self.statistics_mode=='std':
      sigmaE[:,npts_t_long:LEN]=np.std(tmp,axis=-1)
      sigmaE=np.clip(sigmaE,1.e-19,1.e19) # clip the sigmaE to avoid zeros values since FC encounters invalid value if denominator sigmaE is zero
      aveE[:,npts_t_long:LEN]=np.mean(tmp,axis=-1)
    
    # calculate statistics
    if self.statistics_mode=='rms':
      FC=np.abs(E)/(rmsE)
    if self.statistics_mode=='std':
      FC=np.abs(E-aveE)/(sigmaE)
    # reassign FC values for the very beginning couple samples to avoid unreasonable large FC from poor sigmaE
    S=self.t_long;L=int(round(S/dt,0))   # S = 0.1
    for k in range(L):
      FC[:,k]=0
    return FC



#!/usr/bin/env python

import matplotlib.pyplot as plt
import matplotlib
import copy
from obspy.core import *
from .scnl import * 
from .cf_fb import *


class FBPicker():
  """ 
  FBpicker is designed to make picks based on the energy ratio of filtered data.

  This algorithm is scripted by Chen Chen and Guided by Austin A. Holland, June 2015.

  This is a pure python implementation of FBpicker it might be more efficient to wrap the 
  original c-code with swig, but it appears to be more complex than necessary. It uses 
  obspy and numpy to implement the picker routines. 

  FBpicker could be used separately just for picking picks. Also the Associator can 
  follow to associate the picks with phase types.

  """
  
  def __init__(self, t_long = 5, freqmin = 1, corner = 1, perc_taper = 0.1, mode = 'rms', t_ma = 20, nsigma = 6,\
      t_up = 0.78, nr_len = 2, nr_coeff = 2, pol_len = 10, pol_coeff = 10, uncert_coeff = 3):
    
    """ 
    Parameter description:
      t_long       : the time in seconds of moving window to calculate CFn of each bandpass filtered data
      freq_min     : the center frequency of first octave filtering band
      cnr          : corner order of bandpass filtering
      t_ma         : the time in seconds of the moving average window for dynamic threshold
      n_sigma      : controls the level of threshold to trigger potential picks
      t_up         : the time in seconds not allowed consecutive pick in this duration
      mode         : two options: standard deviation (std) or root mean square (rms)
      nr_len       : noise ratio filter window length before and after potential picks used to calculate standard deviation 
      nr_coeff     : control threshold level to determine if remove the pick by comparing std or rms on both sides of each potential pick  
      pol_len      : window length in samples to calculate the standard deviation of waveform before the picks
      pol_coeff    : determine if declare first motion as 'Compression' or 'Dilation' by comparing the first local extreme value after pick and standard deviation in previous window
      uncert_len   : window length in time to calculate the rms of the CF before the picks, we make it as long as t_ma
      uncert_coeff : control the floating level based on the noise of CF
    """

    self.t_long = t_long
    self.freqmin = freqmin
    self.cnr = corner
    self.perc_taper = perc_taper
    self.statistics_mode = mode
    self.t_ma = t_ma
    self.nsigma = nsigma
    self.t_up = t_up
    self.nr_len = nr_len  
    self.nr_coeff = nr_coeff
    self.pol_len = pol_len 
    self.pol_coeff = pol_coeff
    self.uncert_len = self.t_ma
    self.uncert_coeff = uncert_coeff
    

  def picks(self,tr):
    """ 
    Make picks, polarity, snr, and uncertainty.
    """
    
    #tr = trace.detrend('linear')
#       now=time.time()
    summary=FBSummary(self,tr)
#       print "It took %f s to summary" %(time.time()-now)

    # threshold
#       now=time.time()
    threshold=summary.threshold
#       print "It took %f s to threshold" %(time.time()-now)

    # picks
#       now=time.time()
    scnl,picks,trigger,snr=summary.pick_ident()
#       print "It took %f s to ident" %(time.time()-now)

    # uncertainty
#       now=time.time()
    uncertainty=summary.uncertainty()
#       print "It took %f s to uncertainty" %(time.time()-now)
  
    # polarity
#       now=time.time()
    polarity=summary.polarity()
#       print "It took %f s to polarity" %(time.time()-now)
    return scnl,picks,polarity,snr,uncertainty  



class FBSummary():
  """ 
  The class summarizes all bands CFn, threshold level, cleans the false picks, determines uncertainty, polarity 
  and plots band filtered data, statistics for each band, and CF.
  """
  
  def __init__(self,picker, tr):
    self.picker = picker
    self.tr = tr
    self.stats = self.tr.stats
    self.t_long = self.picker.t_long
    self.cnr = self.picker.cnr
    self.perc_taper = self.picker.perc_taper
    self.cf = Ratio(self.tr, self.picker.t_long, self.picker.freqmin, self.cnr, self.perc_taper, self.picker.statistics_mode)
    self.FC = self.cf._statistics()
    self.summary = np.amax(self.FC,axis=0)
    self.thres = self.threshold() 
    self.uncert = self.uncertainty()
    self.pol = self.polarity()  
    
  
  def threshold(self):
    """ 
    Control the threshold level with nsigma.
    """
    dt = self.stats.delta
    npts_Tma = int(round(self.picker.t_ma / dt,0))
    LEN = self.stats.npts
    #print "npts_Tma: ",npts_Tma
    threshold = np.zeros(LEN)
    #threshold[0:npts_Tma] = 1
    threshold[npts_Tma:LEN] = rms(rolling_window(self.summary[0:LEN - 1],npts_Tma), -1) * self.picker.nsigma

    return threshold
  
  def pick_ident(self):
    """ 
    Clean false picks and Make picks.
    """
    
    scnl = SCNL([self.stats.station,self.stats.channel,self.stats.network,self.stats.location])
    dt = self.stats.delta
    npts_Tma = int(round(self.picker.t_ma/dt,0))
    LEN = self.stats.npts
    
    # trigger the earthquakes
    trigger_ptnl_index = np.where(self.summary[npts_Tma:LEN] > self.thres[npts_Tma:LEN])
    trigger_ptnl_index = trigger_ptnl_index + np.array(npts_Tma)
    t = np.arange(0, self.stats.npts/self.stats.sampling_rate, dt) 
    trigger_ptnl = t[trigger_ptnl_index][0]
    
    # clean close picks
    window_t_up = int(round(self.picker.t_up/dt,0))
    trigger_remove1_index = []	
    for i in range(0,len(trigger_ptnl) - 1): # second from last
      if (trigger_ptnl[i+1] - trigger_ptnl[i]) <= window_t_up * dt: # avoid consecutive picking
        trigger_remove1_index.append(i+1)
    # delete close picks
    trigger_ptnl=np.delete(trigger_ptnl, trigger_remove1_index)
    
    # clean_filtering
    trigger_remove2_index = []
    N = self.picker.nr_coeff
    filter_length = self.picker.nr_len    
    for i in range(len(trigger_ptnl)):  
      # determine filter_length for each pick:
      r, R = self.winlen(i, trigger_ptnl, filter_length, t, dt)
      M = min(r, R)
      if N * np.std(self.tr.data[int(round(trigger_ptnl[i] / dt,0)) - M:int(round(trigger_ptnl[i] / dt,0))]) >= np.std(self.tr[int(round(trigger_ptnl[i] / dt,0)):int(round(trigger_ptnl[i] / dt,0)) + M]): 
        trigger_remove2_index.append(i)  
    #delete false picks	
    trigger_ptnl = np.delete(trigger_ptnl, trigger_remove2_index)
    
    # assign potential picks to trigger
    trigger = trigger_ptnl
    
    # really need to be careful copy list or array, since if just copy like A=B, when any element in B changes, A will change as well
    # roll backward for picking
    picks = []
    for i in range(len(trigger)):
      index = int(round(trigger[i] / dt,0))
      while True:
        if self.summary[index] > self.summary[index - 1]:
          index -= 1
        else:
          break
      picks.append(UTCDateTime(self.tr.stats.starttime + round(t[index], 3)))

    # really need to be careful copy list or array, since if just copy like A=B, when any element in B changes, A will change as well
    # roll forward for maximum signal values
    maxes = copy.deepcopy(trigger)
    for i in range(len(trigger)):
       index = int(round(trigger[i] / dt,0))
       while True:
         if self.summary[index] < self.summary[index + 1]:
           index += 1
         else:
           break
       maxes[i] = round(self.summary[index],3)
    
    # really need to be careful copy list or array, since if just copy like A=B, when any element in B changes, A will change as well   
    # Signal noise ration: SNR
    SNR=copy.deepcopy(trigger)

    for i in range(len(picks)):
      index = int(round(trigger[i] / dt,0))
      noise = rms(self.summary[index-npts_Tma:index])
      SNR[i] = round(maxes[i] / noise,1)
    
    return scnl,picks,trigger,SNR
  
  def uncertainty(self):
    """ 
    Uncertainty is determined based on the noise level of CF.
    """
    
    scnl,picks,trigger,SNR = self.pick_ident()
    dt = self.stats.delta
    npts_Tma = int(round(self.picker.t_ma / dt, 0))
    t = np.arange(0, self.stats.npts / self.stats.sampling_rate, dt) 
    pick_uncert = copy.deepcopy(trigger)

    for i in range(len(trigger)):
      r, R=self.winlen(i, trigger, npts_Tma, t, dt)
      index0 = int(round((picks[i] - self.tr.stats.starttime) / dt, 0))
      index = int(round(trigger[i] / dt, 0))
      uncert_level = self.picker.uncert_coeff * rms(self.summary[index0 - npts_Tma:index0])
      while True:
        if self.summary[index] > uncert_level and self.summary[index] > self.summary[index - 1]:
          index -= 1
        else:
          break
      
      # t[index] is the last sample lower than uncert_level, t[index+1] is the first sample higher than uncert_level
      pick_uncert[i] = round(t[index+1] - (picks[i] - self.tr.stats.starttime), 3)    
         
    return pick_uncert
  
  def polarity(self):
    """ 
    Determine polarity for declared picks.
    """
    dt=self.stats.delta
    t = np.arange(0, self.stats.npts/self.stats.sampling_rate, dt) 
    pol=[]
    scnl,picks,trigger,snr=self.pick_ident()
    for i in range(len(picks)):
      index0=int(round((picks[i]-self.tr.stats.starttime)/dt,0))
      index=index0
      
      # roll forward index+=1
      while True:
        if index>=self.stats.npts-1-2:
          break
        elif (self.tr[index+1]-self.tr[index])*(self.tr[index+2]-self.tr[index+1])>0:
          index+=1
        else:
          break
      
      # notice index+1, rolling stop one point before extreme, compare with std to avoid very small 
      if self.tr[index+1] - self.tr[index0] > 0 and abs(self.tr[index+1] - self.tr[index0]) > self.picker.pol_coeff * np.std(self.tr[index0 - self.picker.pol_len: index0]):
        polarity='C'
      elif self.tr[index+1] - self.tr[index0] < 0 and abs(self.tr[index+1] - self.tr[index0]) > self.picker.pol_coeff * np.std(self.tr[index0 - self.picker.pol_len: index0]):
        polarity='D'
      else: 
        polarity=''
      pol.append(polarity)     
    return pol
  
  def winlen(self,index,trigger_ptnl,filter_length,t,dt):
    """ 
    Determine the filter window length. If the time difference between two picks is less 
    than window length, use the picks interval as window.
    """
    i=index
    if len(trigger_ptnl)==1:
      # print 'A'
      if trigger_ptnl[i]<=filter_length:
        r=int(round(trigger_ptnl[i]/dt,0))
      if trigger_ptnl[i]>filter_length:
        r=int(round(filter_length/dt,0))
      if trigger_ptnl[i]+filter_length>=t[-1]: 
        R=int(round((t[-1]-trigger_ptnl[i])/dt,0))
      if trigger_ptnl[i]+filter_length<t[-1]:
        R=int(round(filter_length/dt,0))
    elif len(trigger_ptnl)>1:
      # print 'B'
      if i==0:
        if trigger_ptnl[i]<=filter_length: 
          r=int(round(trigger_ptnl[i]/dt,0)); # print 'a'
        if  trigger_ptnl[i]>filter_length:
          r=int(round(filter_length/dt,0));  # print 'b'
        if (trigger_ptnl[i+1]-trigger_ptnl[i])<=filter_length:
          R=int(round((trigger_ptnl[i+1]-trigger_ptnl[i])/dt,0));  # print 'c'
        if (trigger_ptnl[i+1]-trigger_ptnl[i])>filter_length:
          R=int(round(filter_length/dt,0));  # print 'd'
      elif i>0 and i<len(trigger_ptnl)-1:
        if trigger_ptnl[i]-trigger_ptnl[i-1]<=filter_length:
          r=int(round((trigger_ptnl[i]-trigger_ptnl[i-1])/dt,0));  # print 'e'
        if trigger_ptnl[i]-trigger_ptnl[i-1]>filter_length:
          r=int(round(filter_length/dt,0));  # print 'f'
        if trigger_ptnl[i+1]-trigger_ptnl[i]<=filter_length:
          R=int(round((trigger_ptnl[i+1]-trigger_ptnl[i])/dt,0));  # print 'g'
        if trigger_ptnl[i+1]-trigger_ptnl[i]>filter_length:
          R=int(round(filter_length/dt,0));  # print 'h'
      elif i==len(trigger_ptnl)-1:
        if trigger_ptnl[i]-trigger_ptnl[i-1]<=filter_length:
          r=int(round((trigger_ptnl[i]-trigger_ptnl[i-1])/dt,0));  # print 'i'
        if trigger_ptnl[i]-trigger_ptnl[i-1]>filter_length:
          r=int(round(filter_length/dt,0));  # print 'j'
        if trigger_ptnl[i]+filter_length>t[-1]: 
          R=int(round((t[-1]-trigger_ptnl[i])/dt,0));  # print 'k'
        if trigger_ptnl[i]+filter_length<=t[-1]:
          R=int(round(filter_length/dt,0));  # print 'l'
    return r,R  
    
  def plot_bandfilter(self):
    """ 
    Plot bandpass filtered data BF for each band.
    """
    
    matplotlib.rcParams["axes.labelsize"]="large"
    matplotlib.rcParams["axes.linewidth"]=2.0
    matplotlib.rcParams["xtick.major.size"]=8
    matplotlib.rcParams["ytick.major.size"]=8
    matplotlib.rcParams["ytick.minor.size"]=5
    matplotlib.rcParams["xtick.labelsize"]="large"
    matplotlib.rcParams["ytick.labelsize"]="large"
    
    dt=self.stats.delta
    t = np.arange(0,self.stats.npts/self.stats.sampling_rate,dt)
    n_bands=self.cf._N_bands()
    BF=self.cf.filter()
    fig=plt.figure(figsize=(10,12))
    title='{0} {1} {2}'.format(self.stats.station, self.stats.channel, self.stats.starttime)
    
    # Plot raw data
    ax = plt.subplot(n_bands+1,1,1)
    ax.axes.get_xaxis().set_visible(False)
    ax.plot(t, self.tr, c='gray')
    plt.ylabel('Raw Data')
    plt.title(title)
    
    # Plot filtered data
    st=self.picker.freqmin # for subtitle
    unit='Hz'
    for i in range(n_bands):
        ax = plt.subplot(n_bands+1,1,i+2)
        ax.plot(t, BF[i], c='k')
        subtitle=str(st*2**i)+unit
        ax.text(0.5,0.8,subtitle,horizontalalignment='center',transform=ax.transAxes)
        ax.axes.get_xaxis().set_visible(False)
        plt.ylabel('BF%s'%(i+1))
    ax.axes.get_xaxis().set_visible(True)
    plt.xlabel('Time (s)')
    #fig.savefig('filtering data.pdf')
    #plt.suptitle(title)
    plt.tight_layout()
    plt.show()  
  
  def plot_statistics(self):
    """ 
    Plot CFn for each band.
    """
    
    matplotlib.rcParams["axes.labelsize"]="large"
    matplotlib.rcParams["axes.linewidth"]=2.0
    matplotlib.rcParams["xtick.major.size"]=8
    matplotlib.rcParams["ytick.major.size"]=8
    matplotlib.rcParams["ytick.minor.size"]=5
    matplotlib.rcParams["xtick.labelsize"]="large"
    matplotlib.rcParams["ytick.labelsize"]="large"

    dt = self.stats.delta
    t = np.arange(0, self.stats.npts/self.stats.sampling_rate, dt) 
    scnl,picks,trigger,snr = self.pick_ident()
    fig = plt.figure(figsize=(10,12))
    title='{0} {1} {2}'.format(self.stats.station, self.stats.channel, self.stats.starttime)
    
    # Plot raw data
    n_bands = self.cf._N_bands()
   
    ax = plt.subplot(n_bands+1,1,1)
    ax.plot(t, self.tr, c='gray')
    ax.axes.get_xaxis().set_visible(False)
    # plot picks
    #for i in range(len(picks)):
    #  ax.plot([(picks[i]-self.tr.stats.starttime), (picks[i]-self.tr.stats.starttime)], [min(self.tr),max(self.tr)], 'r--')
    #  ax.text((picks[i]-self.tr.stats.starttime),max(self.tr)-0.3*(max(self.tr)-min(self.tr)),'%s' % (self.pol[i]),color='red')
    plt.ylabel('Raw Data')
    plt.title(title)
    
    # Plot filtered data
    freq=self.picker.freqmin # for subtitle
    unit='Hz'
    for i in range(n_bands):
        ax = plt.subplot(n_bands+1,1,i+2)
        ax.plot(t, self.FC[i], c='k')
        subtitle=str(freq*2**i)+unit
        ax.text(0.5,0.8,subtitle,horizontalalignment='center',transform=ax.transAxes)
        ax.axes.get_xaxis().set_visible(False)
        plt.ylabel('CF%s'%(i+1))
    ax.axes.get_xaxis().set_visible(True)
    plt.xlabel('Time (s)')
    #fig.savefig('CFn_FB.pdf')
    #plt.suptitle(title)
    plt.tight_layout()
    plt.show()
  
  def plot_summary(self):
    """ 
    Plot CF.
    """
    
    matplotlib.rcParams["axes.labelsize"]="large"
    matplotlib.rcParams["axes.linewidth"]=2.0
    matplotlib.rcParams["xtick.major.size"]=8
    matplotlib.rcParams["ytick.major.size"]=8
    matplotlib.rcParams["ytick.minor.size"]=5
    matplotlib.rcParams["xtick.labelsize"]="large"
    matplotlib.rcParams["ytick.labelsize"]="large"
    
    fig=plt.figure(figsize=(12,8))
    dt=self.stats.delta
    t = np.arange(0, self.stats.npts/self.stats.sampling_rate, dt)  
    
    # Plot raw data
    ax = plt.subplot(2,1,1)
    ax.plot(t, self.tr, c='gray')
    plt.ylabel('Raw Data')
    
    # Plot summary
    ax1 = plt.subplot(2,1,2)
    ax1.plot(t,self.summary/np.amax(self.summary),c='k')
    ax1.plot(t,self.thres/np.amax(self.summary),'--',linewidth=2.0,c='k')
    ax1.legend(('Normalized CF','Threshold','Picks'),'upper right', shadow=True, fancybox=True)
    plt.ylabel('Characteristic Function')
    plt.xlabel('Time (s)')
    #plt.xlim([65,69])   # zoom in to see the polarity
    
    # Plot picks
    scnl,picks,trigger,snr=self.pick_ident()
    for i in range(len(picks)):
      ax.plot([(picks[i]-self.tr.stats.starttime),(picks[i]-self.tr.stats.starttime)],[min(self.tr.data),max(self.tr.data)],'k--')
      ax.text((picks[i]-self.tr.stats.starttime),1,'%s' % (self.pol[i]),color='black')
      
    plt.tight_layout()
    plt.show()




from .tables1D import *
from .tt_stations_1D import *
from .func1D import *
import numpy as np
from sqlalchemy.orm import *  # session
from sqlalchemy import create_engine
from scipy.optimize import fmin
from obspy.geodetics import gps2dist_azimuth 
from datetime import *
from operator import itemgetter
from itertools import combinations
import logging
#import time       "from datetime import *" will import time, name space will be overwritten


class LocalAssociator():
  """
  The 1D Associator associate picks with travel time curve of 1D velocity of fixed hypocenter depth.
  """
  def __init__(self, db_assoc, db_tt, max_km = 350, aggregation = 1, aggr_norm = 'L2', assoc_ot_uncert = 3, nsta_declare = 3, cutoff_outlier = 30, loc_uncert_thresh = 0.2):
    """
    Parameters:
    db_assoc: associator database
    db_tt: travel time table database
    max_km: maximum distance of S-P interval in distance
    aggregation: the coefficient multiplied to minimum travel time
    aggr_norm: L2: median; L1: mean
    assoc_ot_uncert: origin time uncertainty window
    nsta_declare: minimum station number to declare a earthquake
    cutoff_outlier: the outlier cut off distance in km
    loc_uncert_thresh: location uncertainty in degree
    """
    
    engine_associator = create_engine(db_assoc, echo=False)
    engine_tt_stations_1D = create_engine(db_tt, echo=False)
    Base.metadata.create_all(engine_associator)               # Base is from the imported tables3D
    BaseTT1D.metadata.create_all(engine_tt_stations_1D)       # BaseTT3D is from the imported tables3D 
    Session1=sessionmaker(bind=engine_associator) # events table  
    Session2=sessionmaker(bind=engine_tt_stations_1D) # traveltime table
    self.assoc_db                 =         Session1() 
    self.tt_stations_db_1D        =         Session2()
    
    self.max_km                   =         max_km
    tmp, d_diff                   =         tt_km(self.tt_stations_db_1D,self.max_km) # From max distance set our maximum travel_time                                            # From max distance set our maximum travel_time
    self.max_tt                   =         tmp.s_tt
    self.max_s_p                  =         tmp.s_p
    self.min_s_p                  =         self.tt_stations_db_1D.query(TTtable1D.s_p).filter(TTtable1D.d_km==0.0).first()[0]
    self.aggregation              =         aggregation
    self.aggr_window              =         self.aggregation * self.min_s_p
    self.aggr_norm                =         aggr_norm         # L1 takes the mean; L2 takes the median
    self.assoc_ot_uncert          =         assoc_ot_uncert                           # Number of seconds between predicted origin times to associate candidate events
    self.nsta_declare             =         nsta_declare      # number observation to declare an evnet
    self.cutoff_outlier           =         cutoff_outlier
    self.loc_uncert_thresh        =         loc_uncert_thresh

    
  def id_candidate_events(self):
    """ 
    Create candidate events.
    """
    #now1 = time.time()
    #############
    # Get all stations with unnassoiated picks
    stations=self.assoc_db.query(Pick.sta).filter(Pick.assoc_id==None).distinct().all()

    for sta, in stations:  # the comma is needed
      picks=self.assoc_db.query(Pick).filter(Pick.sta==sta).filter(Pick.assoc_id==None).order_by(Pick.time).all()
      # Condense picktimes that are within our pick uncertainty value picktimes are python datetime objects
      if stations.index((sta,))==0: #stupid tuple
        counter0=0
        picktimes_new,counter=pick_cluster(self.assoc_db,picks,self.aggr_window,self.aggr_norm,counter0)
      else:
        picktimes_new,counter=pick_cluster(self.assoc_db,picks,self.aggr_window,self.aggr_norm,counter)
      picks_modified=self.assoc_db.query(PickModified).filter(PickModified.sta==sta).filter(PickModified.assoc_id==None).order_by(PickModified.time).all()
      
            
      # Generate all possible candidate events
      for i in range(0,len(picks_modified)-1):
        for j in range(i+1,len(picks_modified)):
          s_p=(picks_modified[j].time-picks_modified[i].time).total_seconds()
          if s_p<=self.max_s_p and s_p>=self.min_s_p:
            tt,tt_uncert=tt_s_p(self.tt_stations_db_1D,s_p) 
            ot=picks_modified[i].time-timedelta(seconds=tt.p_tt)
            new_candidate=Candidate(ot,sta,tt.d_km,tt.delta,picks_modified[i].time,picks_modified[i].id,picks_modified[j].time,picks_modified[j].id)
            self.assoc_db.add(new_candidate)
            self.assoc_db.commit()  
    
    #print 'create candidate time in seconds: ',time.time()-now1, 's'
      
  def associate_candidates(self):
    """ 
    Associate all possible candidate events by comparing the projected origin-times.
    """
    #now2 = time.time()
    
    dt_ot=timedelta(seconds=self.assoc_ot_uncert)
    
    # Query all candidate ots
    candidate_ots=self.assoc_db.query(Candidate).filter(Candidate.assoc_id==None).order_by(Candidate.ot).all()
    L_ots=len(candidate_ots) #; print L_ots
    Array=[]
    for i in range(L_ots):
      cluster=self.assoc_db.query(Candidate).filter(Candidate.assoc_id==None).filter(Candidate.ot>=candidate_ots[i].ot).filter(Candidate.ot<(candidate_ots[i].ot+dt_ot)).order_by(Candidate.ot).all()
      cluster_sta=self.assoc_db.query(Candidate.sta).filter(Candidate.assoc_id==None).filter(Candidate.ot>=candidate_ots[i].ot).filter(Candidate.ot<(candidate_ots[i].ot+dt_ot)).order_by(Candidate.ot).all()
      l_cluster=len(set(cluster_sta))
      Array.append((i,l_cluster,len(cluster)))
    #print Array
    Array.sort(key=itemgetter(1), reverse=True)  #sort Array by l_cluster, notice Array has been changed
    #print Array
    
    #print 'cluster analysis time:', time.time()-now2, 's'
          
    for i in range(len(Array)):
      index=Array[i][0]
      if Array[i][1]>=self.nsta_declare:
        candis=self.assoc_db.query(Candidate).filter(Candidate.assoc_id == None).filter(Candidate.ot >= candidate_ots[index].ot).filter(Candidate.ot < (candidate_ots[index].ot + dt_ot)).order_by(Candidate.ot).all() 
        
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # remove the candidates with the modified picks has been associated
        picks_associated_id=list(set(self.assoc_db.query(PickModified.id).filter(PickModified.assoc_id != None).all()))
        index_candis=[]
        for id, in picks_associated_id:
          for i,candi in enumerate(candis):
            if candi.p_modified_id==id or candi.s_modified_id==id:
              index_candis.append(i)        
        # delete from the end
        if index_candis:
          for j in sorted(set(index_candis), reverse=True):
            del candis[j]
        #print 'candis',candis
        # remove the candidates with the modified picks has been associated
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++        
        # 1D Associator
        # store all necessary parameter in lists    
        radius=[]
        for i,candi in enumerate(candis):
          # pass in the radius for map plotting
          try:
            lon,lat = self.tt_stations_db_1D.query(Station1D.longitude, Station1D.latitude).filter(Station1D.sta == candi.sta).first()
            radius.append((candi.sta, lon, lat, candi.d_km, candi.delta, i)) 
          except:
            print(candi.sta+' did not work')
            pass
    
        cb = self.comb(radius)
        #print 'cb',cb
        
        rms_sort = []
        for i in range(len(cb)):
          radius_cb = cb[i]
          if len(radius_cb) >= self.nsta_declare: # self.nsta_declare has to be greater than or equal to 3
            location=fmin(locating, [lon,lat], radius_cb, disp = 0) # disp = 1 disp : bool, Set to True to print convergence messages.
            residual_minimum=residuals_minimum(location,radius_cb)
            rms_sort.append((location, residual_minimum, i))
            
        # It is possible to have empty rms_sort
        if rms_sort:
          rms_sort.sort(key = itemgetter(1))
          loc, rms, index = rms_sort[0]  # loc is the location before outlier cutoff
          lon = loc[0]
          lat = loc[1]
          matches = cb[index]  # matches is one of combination of radius.append([candi.sta, lon, lat, candi.d_km, candi.delta, i]) 
          #print 'location: ', lat, lon, rms
          #print 'matches',matches
          
          # cut off outlier
          MISMATCHES=[]
          MATCHES_nol, mismatches = outlier_cutoff(matches, loc, self.cutoff_outlier)     # MATCHES_nol is the matches of no outlier, MISMATCHES is the outliers, 
                                                                                          # which are not for locating, only MATCHES_nol are used for locating
          if mismatches:
            MISMATCHES.append(mismatches[0]) 
          while mismatches:
            loc = fmin(locating, [lon,lat], MATCHES_nol, disp = 0)
            MATCHES_nol, mismatches = outlier_cutoff(MATCHES_nol, loc, self.cutoff_outlier)
            if mismatches:
              MISMATCHES.append(mismatches[0])
          #print "MATCHES_nol:",MATCHES_nol,"MISMATCHES:",MISMATCHES
          
          # declare event when nsta and RMS are under control
          nsta = len(MATCHES_nol)
          if nsta >= self.nsta_declare:
            LOC = fmin(locating, (lon,lat), MATCHES_nol, disp = 0)
            LON = round(LOC[0],3)
            LAT = round(LOC[1],3)
            OTS = []
            for i in range(nsta):
              OTS.append(candis[MATCHES_nol[i][5]].ot)
            origintime,ot_unc=datetime_statistics(OTS)
            RMS = residuals_minimum(LOC, MATCHES_nol)
            t_create = datetime.utcnow()
            t_update = datetime.utcnow()
            if RMS <= self.loc_uncert_thresh:
              new_event=Associated(origintime,round(ot_unc,3),LAT,LON,round(RMS,3),nsta,t_create,t_update)    
              self.assoc_db.add(new_event)
              self.assoc_db.flush()
              self.assoc_db.refresh(new_event)
              self.assoc_db.commit()
              event_id=new_event.id
              
              logging.info('event_id: '+ str(event_id))  
              logging.info(str(['ot:', origintime, 'ot_uncert:', ot_unc, 'loc:', LAT,LON, 'loc_uncert:', RMS, 'nsta:', nsta]))
              
              # Associate candidates, picks with the identified event
              for candi in MATCHES_nol:
                candis[candi[5]].set_assoc_id(event_id,self.assoc_db,True)
              self.assoc_db.commit()
                

              # Associate candidates from outliers if the d_km intersect loc_uncert
              if MISMATCHES:
                for i in range(len(MISMATCHES)): 
                  d = gps2dist_azimuth(LAT,LON,MISMATCHES[i][2],MISMATCHES[i][1])[0]/1000
                  r = MISMATCHES[i][3]
                  uncert_km = RMS * np.pi / 180.0 * 6371
                  if abs(d - r) <= uncert_km:
                    candis[MISMATCHES[i][5]].set_assoc_id(event_id,self.assoc_db,False)
              self.assoc_db.commit()

              
        # 1D Associator
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   

      else:
        break
        
  
  
  
  def single_phase(self):
    
    events = self.assoc_db.query(Associated).all()
    for event in events:

      event_id = event.id
      ot = event.ot
      #print event_id,ot
    
    
      # Pick phases that are between origintime and origintime+max_tt
      sta_assoc = []
      for sta, in self.assoc_db.query(PickModified.sta).filter(PickModified.assoc_id==event_id).distinct().all():  # only associated single phase from stations not contribute p and s pairs
        sta_assoc.append(sta)    
    
      # associate single phase
      for sta, in self.assoc_db.query(PickModified.sta).filter(PickModified.assoc_id==None).filter(PickModified.time>ot).filter(PickModified.time<=(ot+timedelta(seconds=self.max_tt))).distinct().all():

        station = self.tt_stations_db_1D.query(Station1D).filter(Station1D.sta==sta).first()
        #print event.latitude,event.longitude,sta,station.latitude,station.longitude
        if event.latitude is None:
          d_km = 1000
        else:
          d_km = gps2dist_azimuth(event.latitude,event.longitude,station.latitude,station.longitude)[0]/1000.
        if (d_km < self.max_km) and (sta not in sta_assoc): # only associated single phase from stations not contribute p and s pairs
          tt,d_diff = tt_km(self.tt_stations_db_1D,d_km)
        
          picks_p = self.assoc_db.query(PickModified).filter(PickModified.sta==sta).filter(PickModified.time>=(ot+timedelta(seconds=tt.p_tt-0.5*self.aggr_window))).filter(PickModified.time<=(ot+timedelta(seconds=tt.p_tt+0.5*self.aggr_window))).all()
          #print 'picks_p: ',picks_p, 'tt.p_tt: ',tt.p_tt
          # if there are more than one modified pick in the aggr_window range, only associate the first modified pick
          if picks_p:
            modi_pick = picks_p[0] # the first modified pick
            modi_pick.phase = 'P'
            modi_pick.assoc_id = event.id
            modi_pick.locate_flag = False
            # Associated all the picks contribute to this single modified picks with assoc_id and phase
            picks=self.assoc_db.query(Pick).filter(Pick.modified_id==modi_pick.id).all()
            for pick in picks:
              pick.phase='P'
              pick.assoc_id=event.id
              pick.locate_flag = False

          picks_s = self.assoc_db.query(PickModified).filter(PickModified.sta==sta).filter(PickModified.time>=(ot+timedelta(seconds=tt.s_tt-0.5*self.aggr_window))).filter(PickModified.time<=(ot+timedelta(seconds=tt.s_tt+0.5*self.aggr_window))).all()  
          # if there are more than one modified pick in the aggr_window range, only associate the first modified pick
          if picks_s:
            modi_pick = picks_s[0] # the first modified pick
            modi_pick.phase = 'S'
            modi_pick.assoc_id = event.id
            modi_pick.locate_flag = None
            # Associated all the picks contribute to this single modified picks with assoc_id and phase
            picks=self.assoc_db.query(Pick).filter(Pick.modified_id==modi_pick.id).all()
            for pick in picks:
              pick.phase='S'
              pick.assoc_id=event.id
              pick.locate_flag = None
      self.assoc_db.commit()

            
  # create the combinations from different stations
  def comb(self,tt):
    L = len(set([item[0] for item in tt]))   # length of the set(sta)
    cb = list(combinations((tt),L))          # combinations of the array, some of them are repeated such as (sta1, sta1, sta2,...)
    
    # remove those combinations of repeated station
    index = []
    for i in range(len(cb)):
      temp = []
      for j in range(L):
        temp.append(cb[i][j][0])
      l = len(set(temp))
      if l < L:
        index.append(i) 
    index.reverse()
    for i in index:
      del cb[i]
 
    # only return combinations of different stations
    return cb   

  
def datetime_statistics(dt_list,norm='L2'):
  """ 
  Calculate the mean and standard deviations in seconds of a list of datetime values.
  """
  offsets=[]
  for dt in dt_list:
    offsets.append((dt-dt_list[0]).total_seconds())
  if norm=='L1':
    mean_offsets=np.mean(offsets)
  elif norm=='L2':
    mean_offsets=np.median(offsets)
  std_offsets=np.std(offsets)
  return dt_list[0]+timedelta(seconds=mean_offsets),std_offsets
  
def pick_cluster(session,picks,pickwindow,pickaveraging_norm,counter):
  """ 
  Cluster picks from different components on the same station.
  """
  #                     |    |                     /\
  #                     |    |                    /  \          /\
  #                     |    | /\      /\        /    \        /  \      /\
  #        _____________|/\__|/  \    /  \      /      \      /    \    /  \  /\_________
  #                     |    |    \  /    \    /        \    /      \  /    \/
  #                     |    |     \/      \  /          \  /        \/
  #                     |    |              \/            \/

  # pickwindow:          ----                                      better to set pickwindow==t_up, t_up is to clean closed picks
  # STA1 E   -----------|----|--------------------|--------------
  # STA1 N   ------------|-------------------------|-------------
  # STA1 Z   -------------|-------------------------|------------
  # stack    -----------|||--|--------------------|||------------  
  # cluster STA1 --------|---|---------------------|-------------  chen highly recommend to use norm=='L2' to lower the effect of outlier, L2 takes median
  # ARGUE: whether only take the median or mean of the picks from different stations? won't count the followings after first one
  # 
  
  picks_new=[]
  # only one pick in picks
  if len(picks)==1:
    cluster=[];cluster.append(picks[0]);cluster_time=[];cluster_time.append(picks[0].time)
    picks[0].modified_id=1+counter # assign modified id to picks
    counter+=1
    pickave,pickstd=datetime_statistics(cluster_time,pickaveraging_norm)
    # append the row to the picks_new, not only the pick time
    picks_new.append(picks[0])
    pick_modified=PickModified(picks[0].sta,picks[0].chan,picks[0].net,picks[0].loc,picks[0].time,picks[0].phase,round(pickstd,3),picks[0].assoc_id)
    session.add(pick_modified)
    session.commit()
    
  # more than one pick in picks
  else:
    j=0
    counter=1+counter
    while True:
      i=j
      cluster=[];cluster.append(picks[i]);cluster_time=[];cluster_time.append(picks[i].time);channel=[];channel.append(picks[i].chan)
      picks[i].modified_id=counter
      while True:
        # cluster picks of different channels; notice that for the closed picks on the same station, those picks behind the first pick could be separated lonely or separated cluster
        if picks[i+1].chan not in channel and (picks[i+1].time-picks[i].time).total_seconds()<pickwindow:
          cluster.append(picks[i+1])
          cluster_time.append(picks[i+1].time)
          channel.append(picks[i+1].chan)
          picks[i+1].modified_id=counter # assign modified id to picks     
          i=i+1
          # make sure do not go over the range limit because j=i+1 below, jump out inner while loop
          if i==len(picks)-1:
            break
        # elif is dealing with the exactly same picks, probably from processing same stream twice
        elif picks[i+1].sta==picks[i].sta and picks[i+1].chan==picks[i].chan and picks[i+1].time==picks[i].time: # and picks[i+1].snr==picks[i].snr and picks[i+1].phase==picks[i].phase and picks[i+1].uncert==picks[i].uncert:
          cluster.append(picks[i+1])
          cluster_time.append(picks[i+1].time)
          channel.append(picks[i+1].chan)
          picks[i+1].modified_id=counter # assign modified id to picks     
          i=i+1
          # make sure do not go over the range limit because j=i+1 below, jump out inner while loop
          if i==len(picks)-1:
            break
        else:
          break
      pickave,pickstd=datetime_statistics(cluster_time,pickaveraging_norm)
      
      # append whole rows to the picks_new, not only the pick time
      for pick in cluster:
        if (pick.time-pickave).total_seconds()>=0:
          break
      picks_new.append(pick)
      pick_modified=PickModified(pick.sta,pick.chan,pick.net,pick.loc,pick.time,pick.phase,round(pickstd,3),pick.assoc_id)
      session.add(pick_modified)
      session.commit()
      # next cluster
      j=i+1
      counter=counter+1
      
      # jump outer while loop and compare last two picks. For the situation that last one is ungrouped, use if statement to add in picks_new
      if j>=len(picks)-1:
        if (picks[-1].time-picks[-2].time).total_seconds()>pickwindow:
          picks_new.append(picks[-1])
          picks[-1].modified_id=counter # assign modified id to picks
          pick_modified=PickModified(picks[-1].sta,picks[-1].chan,picks[-1].net,picks[-1].loc,picks[-1].time,picks[-1].phase,round(pickstd,3),picks[-1].assoc_id)
          session.add(pick_modified)
          session.commit()
        else:
          if picks[-1] in cluster:
            counter-=1
          else:
            picks[-1].modified_id=counter
            pick_modified=PickModified(picks[-1].sta,picks[-1].chan,picks[-1].net,picks[-1].loc,picks[-1].time,picks[-1].phase,round(pickstd,3),picks[-1].assoc_id)
            session.add(pick_modified)
            session.commit()
        break     
  
  return picks_new, counter
 


# locating function is to sum all the distance difference between the iterating guess distance and circle radius; args format is (lon, lat, d_km, sta, delta)
def locating(guess,*args):
#   from obspy.core.util import gps2DistAzimuth
  L=len(args)
  residuals=0
  i=0
  while True:
    # gps2DistAzimuth(lat1, lon1, lat2, lon2) Returns:	(Great circle distance in m, azimuth A->B in degrees, azimuth B->A in degrees)
    residuals=residuals+(gps2dist_azimuth(guess[1],guess[0],args[i][2],args[i][1])[0]/1000*180/(np.pi*6371)-args[i][4])**2
#     np.sqrt((guess[0]-args[i][1])**2+(guess[1]-args[i][2])**2)-args[i][4])**2
    if i==L-1:
      break
    else:                     
      i=i+1
  return np.sqrt(residuals/L)
  
# The only difference with locating function is a * before args. This function return the minimum residual.
def residuals_minimum(location,args):
#   from obspy.core.util import gps2DistAzimuth
  L=len(args)
  residuals=0
  i=0
  while True:
    residuals=residuals+(gps2dist_azimuth(location[1],location[0],args[i][2],args[i][1])[0]/1000*180/(np.pi*6371)-args[i][4])**2
    if i==L-1:
      break
    else:
      i=i+1
  return np.sqrt(residuals/L)
  
# residual function, location is the median lon and lat, args format is (lon, lat, d_km, sta, delta)
def residual(location,args):
  x=gps2dist_azimuth(location[1],location[0],args[2],args[1])[0]/1000*180/(np.pi*6371)-args[4]
  return x


def outlier_cutoff(matches, location, cutoff_outlier):  
  
  # 'tuple' object has no attribute 'remove', the matches passed in is tuple, has to change to list
  matches = list(matches)
  
  res=[]
  for n in range(len(matches)):
    x = residual(location,matches[n])
    res.append(x**2)  # (di - ri)**2 
  m=max(res)
  index=[i for i, j in enumerate(res) if j == m]
  mismatch = []
  for i in index:    # if the min and max have the same absolute value, there will be two index in index list  
    #print 'res',abs(6371*res[i]**0.5*np.pi/180.)
    if abs(6371*res[i]**0.5*np.pi/180.)>cutoff_outlier: 
      mismatch.append(matches[i])
      matches.remove(matches[i])    
  
  # has to return tuple to locate 
  return tuple(matches), tuple(mismatch)

    



#! /usr/bin/env python

# Automatic picking of seismic waves using Generalized Phase Detection 
# See http://scedc.caltech.edu/research-tools/deeplearning.html for more info
#
# Ross et al. (2018), Generalized Seismic Phase Detection with Deep Learning,
#                     Bull. Seismol. Soc. Am., doi:10.1785/0120180080
#                                              
# Author: Zachary E. Ross (2018)                
# Contact: zross@gps.caltech.edu                        
# Website: http://www.seismolab.caltech.edu/ross_z.html         

import string
import time
import argparse as ap
import sys
import os

import numpy as np
import obspy.core as oc
import keras
from keras.models import Sequential
from keras.layers import Dense, Dropout, Activation, Flatten
from keras.layers import Conv1D, MaxPooling1D
from keras import losses
from keras.models import model_from_json
import tensorflow as tf
import matplotlib as mpl
import pylab as plt
#mpl.rcParams['pdf.fonttype'] = 42
physical_devices = tf.config.list_physical_devices('GPU')
try:
  tf.config.experimental.set_memory_growth(physical_devices[0], True)
except:
  # Invalid device or cannot modify virtual devices once initialized.
  pass
#####################
# Hyperparameters
min_proba = 0.994 # Minimum softmax probability for phase detection
freq_min = 3.0
freq_max = 20.0
filter_data = True
decimate_data = True # If false, assumes data is already 100 Hz samprate
n_shift = 10 # Number of samples to shift the sliding window at a time
n_gpu = 1 # Number of GPUs to use (if any)
#####################
batch_size = 1000*3

half_dur = 2.00
only_dt = 0.01
n_win = int(half_dur/only_dt)
n_feat = 2*n_win

#-------------------------------------------------------------

def sliding_window(data, size, stepsize=1, padded=False, axis=-1, copy=True):
    """
    Calculate a sliding window over a signal
    Parameters
    ----------
    data : numpy array
        The array to be slided over.
    size : int
        The sliding window size
    stepsize : int
        The sliding window stepsize. Defaults to 1.
    axis : int
        The axis to slide over. Defaults to the last axis.
    copy : bool
        Return strided array as copy to avoid sideffects when manipulating the
        output array.
    Returns
    -------
    data : numpy array
        A matrix where row in last dimension consists of one instance
        of the sliding window.
    Notes
    -----
    - Be wary of setting `copy` to `False` as undesired sideffects with the
      output values may occurr.
    Examples
    --------
    >>> a = numpy.array([1, 2, 3, 4, 5])
    >>> sliding_window(a, size=3)
    array([[1, 2, 3],
           [2, 3, 4],
           [3, 4, 5]])
    >>> sliding_window(a, size=3, stepsize=2)
    array([[1, 2, 3],
           [3, 4, 5]])
    See Also
    --------
    pieces : Calculate number of pieces available by sliding
    """
    if axis >= data.ndim:
        raise ValueError(
            "Axis value out of range"
        )

    if stepsize < 1:
        raise ValueError(
            "Stepsize may not be zero or negative"
        )

    if size > data.shape[axis]:
        raise ValueError(
            "Sliding window size may not exceed size of selected axis"
        )

    shape = list(data.shape)
    shape[axis] = np.floor(data.shape[axis] / stepsize - size / stepsize + 1).astype(int)
    shape.append(size)

    strides = list(data.strides)
    strides[axis] *= stepsize
    strides.append(data.strides[axis])

    strided = np.lib.stride_tricks.as_strided(
        data, shape=shape, strides=strides
    )

    if copy:
        return strided.copy()
    else:
        return strided

if __name__ == "__main__":
    parser = ap.ArgumentParser(
        prog='gpd_predict.py',
        description='Automatic picking of seismic waves using'
                    'Generalized Phase Detection')
    parser.add_argument(
        '-I',
        type=str,
        default=None,
        help='Input file')
    parser.add_argument(
        '-O',
        type=str,
        default=None,
        help='Output file')
    parser.add_argument(
        '-P',
        default=True,
        action='store_false',
        help='Suppress plotting output')
    parser.add_argument(
        '-V',
        default=False,
        action='store_true',
        help='verbose')
    parser.add_argument(
        '-F',
        type=str,
        default=None,
        help='path where GPD lives')
    args = parser.parse_args()

    plot = args.P

    # Reading in input file
    fdir = []
    evid = []
    staid = []
    with open(args.I) as f:
        for line in f:
            tmp = line.split()
            fdir.append([tmp[0], tmp[1], tmp[2]])
    nsta = len(fdir)

    # load json and create model
    pathjson = args.F+'/model_pol.json'
    json_file = open(pathjson, 'r')
    loaded_model_json = json_file.read()
    json_file.close()
    model = model_from_json(loaded_model_json, custom_objects={'tf':tf})

    # load weights into new model
    model.load_weights(args.F+"/model_pol_best.hdf5")
    print("Loaded model from disk")

    if n_gpu > 1:
        from keras.utils import multi_gpu_model
        model = multi_gpu_model(model, gpus=n_gpu)

    ofile = open(args.O, 'w')

    for i in range(nsta):
        fname = fdir[i][0].split("/")
        if not os.path.isfile(fdir[i][0]):
            print("%s doesn't exist, skipping" % fdir[i][0])
            continue
        if not os.path.isfile(fdir[i][1]):
            print("%s doesn't exist, skipping" % fdir[i][1])
            continue
        if not os.path.isfile(fdir[i][2]):
            print("%s doesn't exist, skipping" % fdir[i][2])
            continue
        st = oc.Stream()
        st += oc.read(fdir[i][0])
        st += oc.read(fdir[i][1])
        st += oc.read(fdir[i][2])
        st.merge()
        print(st)
        for tr in st:
            if isinstance(tr.data, np.ma.masked_array):
                tr.data = tr.data.filled()

        latest_start = np.max([x.stats.starttime for x in st])
        earliest_stop = np.min([x.stats.endtime for x in st])
        st.trim(latest_start, earliest_stop)

        st.detrend(type='linear')
        if filter_data:
            st.filter(type='bandpass', freqmin=freq_min, freqmax=freq_max)
        if decimate_data:
            st.interpolate(100.0)
        chan = st[0].stats.channel
        sr = st[0].stats.sampling_rate

        dt = st[0].stats.delta
        net = st[0].stats.network
        sta = st[0].stats.station
        chan = st[0].stats.channel
        latest_start = np.max([x.stats.starttime for x in st])
        earliest_stop = np.min([x.stats.endtime for x in st])
        st.trim(latest_start, earliest_stop)
        if args.V:
            print("Reshaping data matrix for sliding window")
        tt = (np.arange(0, st[0].data.size, n_shift) + n_win) * dt
        tt_i = np.arange(0, st[0].data.size, n_shift) + n_feat
        #tr_win = np.zeros((tt.size, n_feat, 3))
        sliding_N = sliding_window(st[0].data, n_feat, stepsize=n_shift)
        sliding_E = sliding_window(st[1].data, n_feat, stepsize=n_shift)
        sliding_Z = sliding_window(st[2].data, n_feat, stepsize=n_shift)
        tr_win = np.zeros((sliding_N.shape[0], n_feat, 3))
        tr_win[:,:,0] = sliding_N
        tr_win[:,:,1] = sliding_E
        tr_win[:,:,2] = sliding_Z
        tr_win = tr_win / np.max(np.abs(tr_win), axis=(1,2))[:,None,None]
        tt = tt[:tr_win.shape[0]]
        tt_i = tt_i[:tr_win.shape[0]]

        if args.V:
            ts = model.predict(tr_win, verbose=True, batch_size=batch_size)
        else:
            ts = model.predict(tr_win, verbose=False, batch_size=batch_size)

        prob_S = ts[:,1]
        prob_P = ts[:,0]
        prob_N = ts[:,2]

        from obspy.signal.trigger import trigger_onset
        trigs = trigger_onset(prob_P, min_proba, 0.1)
        p_picks = []
        s_picks = []
        for trig in trigs:
            if trig[1] == trig[0]:
                continue
            pick = np.argmax(ts[trig[0]:trig[1], 0])+trig[0]
            stamp_pick = st[0].stats.starttime + tt[pick]
            #chan_pick = st[0].stats.channel
            p_picks.append(stamp_pick)
            ofile.write("%s %s HHZ P %s\n" % (net, sta, stamp_pick.isoformat()))

        trigs = trigger_onset(prob_S, min_proba, 0.1)
        for trig in trigs:
            if trig[1] == trig[0]:
                continue
            pick = np.argmax(ts[trig[0]:trig[1], 1])+trig[0]
            stamp_pick = st[0].stats.starttime + tt[pick]
            chan_pick = st[0].stats.channel
            s_picks.append(stamp_pick)
            ofile.write("%s %s HHE S %s\n" % (net, sta, stamp_pick.isoformat()))

        if plot:
            fig = plt.figure(figsize=(8, 12))
            ax = []
            ax.append(fig.add_subplot(4,1,1))
            ax.append(fig.add_subplot(4,1,2,sharex=ax[0],sharey=ax[0]))
            ax.append(fig.add_subplot(4,1,3,sharex=ax[0],sharey=ax[0]))
            ax.append(fig.add_subplot(4,1,4,sharex=ax[0]))
            for i in range(3):
                ax[i].plot(np.arange(st[i].data.size)*dt, st[i].data, c='k', \
                           lw=0.5)
            ax[3].plot(tt, ts[:,0], c='r', lw=0.5)
            ax[3].plot(tt, ts[:,1], c='b', lw=0.5)
            for p_pick in p_picks:
                for i in range(3):
                    ax[i].axvline(p_pick-st[0].stats.starttime, c='r', lw=0.5)
            for s_pick in s_picks:
                for i in range(3):
                    ax[i].axvline(s_pick-st[0].stats.starttime, c='b', lw=0.5)
            plt.tight_layout()
            plt.show()
    ofile.close()
